from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class ProductPackageDimensions(BaseModel):
    """Describes the physical size and weight of the product. Contains the following fields:
    * height
    * length
    * weight
    * width
    These fields are represented as numbers, but it is the responsibility of the merchant to define and interpret the relevant units of length and weight.


    :param height: NA, defaults to None
    :type height: float, optional
    :param length: NA, defaults to None
    :type length: float, optional
    :param weight: NA, defaults to None
    :type weight: float, optional
    :param width: NA, defaults to None
    :type width: float, optional
    """

    def __init__(
        self,
        height: float = None,
        length: float = None,
        weight: float = None,
        width: float = None,
    ):
        """Describes the physical size and weight of the product. Contains the following fields:
        * height
        * length
        * weight
        * width
        These fields are represented as numbers, but it is the responsibility of the merchant to define and interpret the relevant units of length and weight.


        :param height: NA, defaults to None
        :type height: float, optional
        :param length: NA, defaults to None
        :type length: float, optional
        :param weight: NA, defaults to None
        :type weight: float, optional
        :param width: NA, defaults to None
        :type width: float, optional
        """
        self.height = self._define_number("height", height, nullable=True)
        self.length = self._define_number("length", length, nullable=True)
        self.weight = self._define_number("weight", weight, nullable=True)
        self.width = self._define_number("width", width, nullable=True)



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .inline_response_200_73_data_elements import InlineResponse200_73DataElements


@JsonMap({})
class InlineResponse200_73DataComplianceLevels(BaseModel):
    """InlineResponse200_73DataComplianceLevels

    :param level: level, defaults to None
    :type level: float, optional
    :param elements: elements, defaults to None
    :type elements: List[InlineResponse200_73DataElements], optional
    """

    def __init__(
        self,
        level: float = None,
        elements: List[InlineResponse200_73DataElements] = None,
    ):
        """InlineResponse200_73DataComplianceLevels

        :param level: level, defaults to None
        :type level: float, optional
        :param elements: elements, defaults to None
        :type elements: List[InlineResponse200_73DataElements], optional
        """
        self.level = self._define_number("level", level, nullable=True)
        self.elements = self._define_list(elements, InlineResponse200_73DataElements)



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .list_countries_response_languages import ListCountriesResponseLanguages


@JsonMap({})
class ListCountriesResponse(BaseModel):
    """ListCountriesResponse

    :param languages: List of the supported countries., defaults to None
    :type languages: ListCountriesResponseLanguages, optional
    """

    def __init__(self, languages: ListCountriesResponseLanguages = None):
        """ListCountriesResponse

        :param languages: List of the supported countries., defaults to None
        :type languages: ListCountriesResponseLanguages, optional
        """
        self.languages = self._define_object(languages, ListCountriesResponseLanguages)



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .inline_response_200_90_data_organization_details_merchant_customer_support import (
    InlineResponse200_90DataOrganizationDetailsMerchantCustomerSupport,
)


@JsonMap({})
class InlineResponse200_90DataOrganizationDetails(BaseModel):
    """InlineResponse200_90DataOrganizationDetails

    :param merchant_color: merchant_color, defaults to None
    :type merchant_color: str, optional
    :param merchant_website: merchant_website, defaults to None
    :type merchant_website: str, optional
    :param merchant_logo: merchant_logo, defaults to None
    :type merchant_logo: str, optional
    :param merchant_design: merchant_design, defaults to None
    :type merchant_design: str, optional
    :param merchant_language: merchant_language, defaults to None
    :type merchant_language: str, optional
    :param merchant_alias: merchant_alias, defaults to None
    :type merchant_alias: str, optional
    :param merchant_customer_support: merchant_customer_support, defaults to None
    :type merchant_customer_support: InlineResponse200_90DataOrganizationDetailsMerchantCustomerSupport, optional
    """

    def __init__(
        self,
        merchant_color: str = None,
        merchant_website: str = None,
        merchant_logo: str = None,
        merchant_design: str = None,
        merchant_language: str = None,
        merchant_alias: str = None,
        merchant_customer_support: InlineResponse200_90DataOrganizationDetailsMerchantCustomerSupport = None,
    ):
        """InlineResponse200_90DataOrganizationDetails

        :param merchant_color: merchant_color, defaults to None
        :type merchant_color: str, optional
        :param merchant_website: merchant_website, defaults to None
        :type merchant_website: str, optional
        :param merchant_logo: merchant_logo, defaults to None
        :type merchant_logo: str, optional
        :param merchant_design: merchant_design, defaults to None
        :type merchant_design: str, optional
        :param merchant_language: merchant_language, defaults to None
        :type merchant_language: str, optional
        :param merchant_alias: merchant_alias, defaults to None
        :type merchant_alias: str, optional
        :param merchant_customer_support: merchant_customer_support, defaults to None
        :type merchant_customer_support: InlineResponse200_90DataOrganizationDetailsMerchantCustomerSupport, optional
        """
        self.merchant_color = self._define_str(
            "merchant_color", merchant_color, nullable=True
        )
        self.merchant_website = self._define_str(
            "merchant_website", merchant_website, nullable=True
        )
        self.merchant_logo = self._define_str(
            "merchant_logo", merchant_logo, nullable=True
        )
        self.merchant_design = self._define_str(
            "merchant_design", merchant_design, nullable=True
        )
        self.merchant_language = self._define_str(
            "merchant_language", merchant_language, nullable=True
        )
        self.merchant_alias = self._define_str(
            "merchant_alias", merchant_alias, nullable=True
        )
        self.merchant_customer_support = self._define_object(
            merchant_customer_support,
            InlineResponse200_90DataOrganizationDetailsMerchantCustomerSupport,
        )



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .escrow_response_escrow_releases_data import EscrowResponseEscrowReleasesData


@JsonMap({})
class EscrowResponseEscrowReleases(BaseModel):
    """Array of objects that describe individual releases.

    :param data: Array of objects that describe individual escrow releases., defaults to None
    :type data: EscrowResponseEscrowReleasesData, optional
    :param has_more: Indicates that the number of escrow releases is greater than the number returned in the response., defaults to None
    :type has_more: bool, optional
    :param total_count: Number of escrow releases., defaults to None
    :type total_count: float, optional
    :param url: URL for the record of all escrow releases for this payment., defaults to None
    :type url: str, optional
    """

    def __init__(
        self,
        data: EscrowResponseEscrowReleasesData = None,
        has_more: bool = None,
        total_count: float = None,
        url: str = None,
    ):
        """Array of objects that describe individual releases.

        :param data: Array of objects that describe individual escrow releases., defaults to None
        :type data: EscrowResponseEscrowReleasesData, optional
        :param has_more: Indicates that the number of escrow releases is greater than the number returned in the response., defaults to None
        :type has_more: bool, optional
        :param total_count: Number of escrow releases., defaults to None
        :type total_count: float, optional
        :param url: URL for the record of all escrow releases for this payment., defaults to None
        :type url: str, optional
        """
        self.data = self._define_object(data, EscrowResponseEscrowReleasesData)
        self.has_more = has_more
        self.total_count = self._define_number(
            "total_count", total_count, nullable=True
        )
        self.url = self._define_str("url", url, nullable=True)



from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


class BeneficiaryBeneficiaryIdBodyGender(Enum):
    """An enumeration representing different categories.

    :cvar MALE: "male"
    :vartype MALE: str
    :cvar FEMALE: "female"
    :vartype FEMALE: str
    :cvar OTHER: "other"
    :vartype OTHER: str
    :cvar NOTAPPLICABLE: "not_applicable"
    :vartype NOTAPPLICABLE: str
    """

    MALE = "male"
    FEMALE = "female"
    OTHER = "other"
    NOTAPPLICABLE = "not_applicable"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                BeneficiaryBeneficiaryIdBodyGender._member_map_.values(),
            )
        )


class BeneficiaryBeneficiaryIdBodyIdentificationType(Enum):
    """An enumeration representing different categories.

    :cvar DRIVERSLICENSE: "drivers_license"
    :vartype DRIVERSLICENSE: str
    :cvar IDENTIFICATIONID: "identification_id"
    :vartype IDENTIFICATIONID: str
    :cvar INTERNATIONALPASSPORT: "international_passport"
    :vartype INTERNATIONALPASSPORT: str
    :cvar RESIDENCEPERMIT: "residence_permit"
    :vartype RESIDENCEPERMIT: str
    :cvar SOCIALSECURITY: "social_security"
    :vartype SOCIALSECURITY: str
    :cvar WORKPERMIT: "work_permit"
    :vartype WORKPERMIT: str
    """

    DRIVERSLICENSE = "drivers_license"
    IDENTIFICATIONID = "identification_id"
    INTERNATIONALPASSPORT = "international_passport"
    RESIDENCEPERMIT = "residence_permit"
    SOCIALSECURITY = "social_security"
    WORKPERMIT = "work_permit"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                BeneficiaryBeneficiaryIdBodyIdentificationType._member_map_.values(),
            )
        )


@JsonMap({})
class BeneficiaryBeneficiaryIdBody(BaseModel):
    """BeneficiaryBeneficiaryIdBody

    :param address: Beneficiary's street address including the build number., defaults to None
    :type address: str, optional
    :param city: City of the beneficiary., defaults to None
    :type city: str, optional
    :param company_name: Name of the beneficiary company. Relevant when `entity_type` is **company**., defaults to None
    :type company_name: str, optional
    :param country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
    :type country: str, optional
    :param country_of_incorporation: The country where the company was registered. Two-letter ISO 3166-1 ALPHA-2 code. Relevant when `entity_type` is **company**., defaults to None
    :type country_of_incorporation: str, optional
    :param date_of_birth: Date of birth of the individual. Format: DD/MM/YYYY. Relevant when `entity_type` is **individual**., defaults to None
    :type date_of_birth: str, optional
    :param date_of_incorporation: The date when the company was registered. Format: DD/MM/YYYY. Relevant when `entity_type` is **company**., defaults to None
    :type date_of_incorporation: str, optional
    :param default_payout_method_type: The type of payout method for the beneficiary. The two-letter prefix must match the beneficiary country code., defaults to None
    :type default_payout_method_type: str, optional
    :param first_name: First name of the beneficiary. Relevant when `entity_type` is **individual**., defaults to None
    :type first_name: str, optional
    :param gender: Gender of the individual. Relevant when `entity_type` is **individual**., defaults to None
    :type gender: BeneficiaryBeneficiaryIdBodyGender, optional
    :param identification_type: Type of identification document for the beneficiary. When `entity_type` is **company**, this field must be**company_registered_number**. When `entity_type` is **individual**:, defaults to None
    :type identification_type: BeneficiaryBeneficiaryIdBodyIdentificationType, optional
    :param identification_value: Identification number on the document mentioned in `identification_type`., defaults to None
    :type identification_value: str, optional
    :param last_name: Family name of the beneficiary. Relevant when `entity_type` is **individual**. Required when `entity_type` is **individual**., defaults to None
    :type last_name: str, optional
    :param merchant_reference_id: ID defined by the client., defaults to None
    :type merchant_reference_id: str, optional
    :param nationality: The citizenship of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code for the country. To determine the code for a country, see 'List Countries'. Relevant when `entity_type` is **individual**., defaults to None
    :type nationality: str, optional
    """

    def __init__(
        self,
        address: str = None,
        city: str = None,
        company_name: str = None,
        country: str = None,
        country_of_incorporation: str = None,
        date_of_birth: str = None,
        date_of_incorporation: str = None,
        default_payout_method_type: str = None,
        first_name: str = None,
        gender: BeneficiaryBeneficiaryIdBodyGender = None,
        identification_type: BeneficiaryBeneficiaryIdBodyIdentificationType = None,
        identification_value: str = None,
        last_name: str = None,
        merchant_reference_id: str = None,
        nationality: str = None,
    ):
        """BeneficiaryBeneficiaryIdBody

        :param address: Beneficiary's street address including the build number., defaults to None
        :type address: str, optional
        :param city: City of the beneficiary., defaults to None
        :type city: str, optional
        :param company_name: Name of the beneficiary company. Relevant when `entity_type` is **company**., defaults to None
        :type company_name: str, optional
        :param country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
        :type country: str, optional
        :param country_of_incorporation: The country where the company was registered. Two-letter ISO 3166-1 ALPHA-2 code. Relevant when `entity_type` is **company**., defaults to None
        :type country_of_incorporation: str, optional
        :param date_of_birth: Date of birth of the individual. Format: DD/MM/YYYY. Relevant when `entity_type` is **individual**., defaults to None
        :type date_of_birth: str, optional
        :param date_of_incorporation: The date when the company was registered. Format: DD/MM/YYYY. Relevant when `entity_type` is **company**., defaults to None
        :type date_of_incorporation: str, optional
        :param default_payout_method_type: The type of payout method for the beneficiary. The two-letter prefix must match the beneficiary country code., defaults to None
        :type default_payout_method_type: str, optional
        :param first_name: First name of the beneficiary. Relevant when `entity_type` is **individual**., defaults to None
        :type first_name: str, optional
        :param gender: Gender of the individual. Relevant when `entity_type` is **individual**., defaults to None
        :type gender: BeneficiaryBeneficiaryIdBodyGender, optional
        :param identification_type: Type of identification document for the beneficiary. When `entity_type` is **company**, this field must be**company_registered_number**. When `entity_type` is **individual**:, defaults to None
        :type identification_type: BeneficiaryBeneficiaryIdBodyIdentificationType, optional
        :param identification_value: Identification number on the document mentioned in `identification_type`., defaults to None
        :type identification_value: str, optional
        :param last_name: Family name of the beneficiary. Relevant when `entity_type` is **individual**. Required when `entity_type` is **individual**., defaults to None
        :type last_name: str, optional
        :param merchant_reference_id: ID defined by the client., defaults to None
        :type merchant_reference_id: str, optional
        :param nationality: The citizenship of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code for the country. To determine the code for a country, see 'List Countries'. Relevant when `entity_type` is **individual**., defaults to None
        :type nationality: str, optional
        """
        self.address = self._define_str("address", address, nullable=True)
        self.city = self._define_str("city", city, nullable=True)
        self.company_name = self._define_str(
            "company_name", company_name, nullable=True
        )
        self.country = self._define_str("country", country, nullable=True)
        self.country_of_incorporation = self._define_str(
            "country_of_incorporation", country_of_incorporation, nullable=True
        )
        self.date_of_birth = self._define_str(
            "date_of_birth", date_of_birth, nullable=True
        )
        self.date_of_incorporation = self._define_str(
            "date_of_incorporation", date_of_incorporation, nullable=True
        )
        self.default_payout_method_type = self._define_str(
            "default_payout_method_type", default_payout_method_type, nullable=True
        )
        self.first_name = self._define_str("first_name", first_name, nullable=True)
        self.gender = (
            self._enum_matching(
                gender, BeneficiaryBeneficiaryIdBodyGender.list(), "gender"
            )
            if gender
            else None
        )
        self.identification_type = (
            self._enum_matching(
                identification_type,
                BeneficiaryBeneficiaryIdBodyIdentificationType.list(),
                "identification_type",
            )
            if identification_type
            else None
        )
        self.identification_value = self._define_str(
            "identification_value", identification_value, nullable=True
        )
        self.last_name = self._define_str("last_name", last_name, nullable=True)
        self.merchant_reference_id = self._define_str(
            "merchant_reference_id", merchant_reference_id, nullable=True
        )
        self.nationality = self._define_str("nationality", nationality, nullable=True)



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .fx_fee import FxFee
from .transaction_fee import TransactionFee


@JsonMap({})
class PayoutFees(BaseModel):
    """PayoutFees

    :param fx_fee: Describes the fees for processing the currency exchange. Relevant to payouts with FX., defaults to None
    :type fx_fee: FxFee, optional
    :param gross_fees: The total gross fees for the transaction, in units defined by currency_code. Relevant to responses., defaults to None
    :type gross_fees: float, optional
    :param net_fees: The total net fees for the transaction, in units defined by merchant_requested_currency. Relevant to responses., defaults to None
    :type net_fees: float, optional
    :param transaction_fee: Describes the fee for processing the transaction., defaults to None
    :type transaction_fee: TransactionFee, optional
    """

    def __init__(
        self,
        fx_fee: FxFee = None,
        gross_fees: float = None,
        net_fees: float = None,
        transaction_fee: TransactionFee = None,
    ):
        """PayoutFees

        :param fx_fee: Describes the fees for processing the currency exchange. Relevant to payouts with FX., defaults to None
        :type fx_fee: FxFee, optional
        :param gross_fees: The total gross fees for the transaction, in units defined by currency_code. Relevant to responses., defaults to None
        :type gross_fees: float, optional
        :param net_fees: The total net fees for the transaction, in units defined by merchant_requested_currency. Relevant to responses., defaults to None
        :type net_fees: float, optional
        :param transaction_fee: Describes the fee for processing the transaction., defaults to None
        :type transaction_fee: TransactionFee, optional
        """
        self.fx_fee = self._define_object(fx_fee, FxFee)
        self.gross_fees = self._define_number("gross_fees", gross_fees, nullable=True)
        self.net_fees = self._define_number("net_fees", net_fees, nullable=True)
        self.transaction_fee = self._define_object(transaction_fee, TransactionFee)



from typing import Union
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.one_of_base_model import OneOfBaseModel


class Field1ConditionsThresholdValueGuard(OneOfBaseModel):
    class_list = {"str": str, "List[str]": List[str]}


Field1ConditionsThresholdValue = Union[str, List[str]]


@JsonMap({})
class Field1Conditions(BaseModel):
    """Field1Conditions

    :param description: Description of the condition., defaults to None
    :type description: str, optional
    :param element_name: The name of a field, including the path. The field is the first operand of the condition., defaults to None
    :type element_name: str, optional
    :param operator: A symbol representing the operator of the condition. String starting with $. The operator determines the relationship between the operands., defaults to None
    :type operator: str, optional
    :param threshold_value: des One or more possible values of the element_name field. The second operand of the condition., defaults to None
    :type threshold_value: Field1ConditionsThresholdValue, optional
    """

    def __init__(
        self,
        description: str = None,
        element_name: str = None,
        operator: str = None,
        threshold_value: Field1ConditionsThresholdValue = None,
    ):
        """Field1Conditions

        :param description: Description of the condition., defaults to None
        :type description: str, optional
        :param element_name: The name of a field, including the path. The field is the first operand of the condition., defaults to None
        :type element_name: str, optional
        :param operator: A symbol representing the operator of the condition. String starting with $. The operator determines the relationship between the operands., defaults to None
        :type operator: str, optional
        :param threshold_value: des One or more possible values of the element_name field. The second operand of the condition., defaults to None
        :type threshold_value: Field1ConditionsThresholdValue, optional
        """
        self.description = self._define_str("description", description, nullable=True)
        self.element_name = self._define_str(
            "element_name", element_name, nullable=True
        )
        self.operator = self._define_str("operator", operator, nullable=True)
        self.threshold_value = Field1ConditionsThresholdValueGuard.return_one_of(
            threshold_value
        )



from enum import Enum


class HostedPageStatus(Enum):
    """An enumeration representing different categories.

    :cvar NEW: "NEW"
    :vartype NEW: str
    :cvar DON: "DON"
    :vartype DON: str
    :cvar EXP: "EXP"
    :vartype EXP: str
    """

    NEW = "NEW"
    DON = "DON"
    EXP = "EXP"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, HostedPageStatus._member_map_.values()))



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .merchant_customer_support import MerchantCustomerSupport
from .hosted_page_status import HostedPageStatus
from .hosted_page_additional_response_cart_items import (
    HostedPageAdditionalResponseCartItems,
)
from .hosted_page_additional_response_custom_elements import (
    HostedPageAdditionalResponseCustomElements,
)
from .payment import Payment


@JsonMap({"id_": "id"})
class CheckoutPageResponse(BaseModel):
    """CheckoutPageResponse

    :param language: Determines the default language of the hosted page. When this parameter is null, the language of the user's browser is used. If the language of the user's browser cannot be determined, the default language is English, defaults to None
    :type language: str, optional
    :param merchant_alias: Reserved, the default name is Rapyd, defaults to None
    :type merchant_alias: str, optional
    :param merchant_color: Color of the call-to-action (CTA) button on the hosted page. To configure this field, use the Client Portal, defaults to None
    :type merchant_color: str, optional
    :param merchant_customer_support: Contains details of the client’s customer support. To configure these fields, use the Client Portal., defaults to None
    :type merchant_customer_support: MerchantCustomerSupport, optional
    :param merchant_logo: URL for the image of the client's logo. To configure this field, use the Client Portal, defaults to None
    :type merchant_logo: str, optional
    :param merchant_website: The URL where the customer is redirected after exiting the hosted page. Relevant when one or both of the following fields is unset: \<BR\>* `cancel_url` \<BR\>* `complete_url`.\<BR\> To configure this field, use the Client Portal, defaults to None
    :type merchant_website: str, optional
    :param redirect_url: URL of the hosted page that is shown to the customer., defaults to None
    :type redirect_url: str, optional
    :param merchant_terms: URL for the client's terms and conditions. To configure this field, use the Client Portal, defaults to None
    :type merchant_terms: str, optional
    :param cancel_url: URL where the customer is redirected after pressing Back to Website to exit the hosted page. This URL overrides the merchant_website URL. Does not support localhost URLs., defaults to None
    :type cancel_url: str, optional
    :param complete_url: URL where the customer is redirected after pressing Close to exit the hosted page. This URL overrides the merchant_website URL. Does not support localhost URLs., defaults to None
    :type complete_url: str, optional
    :param merchant_privacy_policy: URL for the terms and conditions of the agreement between the client and the client’s customers. To configure this field, use the Client Portal., defaults to None
    :type merchant_privacy_policy: str, optional
    :param page_expiration: Length of time for the payment to be completed after it is created, measured in seconds. When both expiration and payment_expiration are set, the payment expires at the earlier time., defaults to None
    :type page_expiration: float, optional
    :param status: Status of the hosted page. One of the following: NEW - The hosted page was created. DON - Done. The card was added to the customer profile. EXP - The hosted page expired. , defaults to None
    :type status: HostedPageStatus, optional
    :param account_funding_transaction: Details of an account funding transaction (AFT), which transfers funds from a card to a cardholder's wallet., defaults to None
    :type account_funding_transaction: dict, optional
    :param amount: The amount of the payment, in units of the currency defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015. If the amount is a whole number, use an integer and not a decimal., defaults to None
    :type amount: float, optional
    :param cancel_checkout_url: URL where the customer is redirected after pressing Back to Website to exit the hosted page. This URL overrides the merchant_website URL. Does not support localhost URLs., defaults to None
    :type cancel_checkout_url: str, optional
    :param cart_items: Describes the cart items that the customer is purchasing. These items are displayed at the checkout page., defaults to None
    :type cart_items: HostedPageAdditionalResponseCartItems, optional
    :param complete_checkout_url: URL where the customer is redirected after pressing Finish to exit the hosted page. This URL overrides the merchant_website URL. Does not support localhost URLs., defaults to None
    :type complete_checkout_url: str, optional
    :param country: country, defaults to None
    :type country: str, optional
    :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
    :type currency: str, optional
    :param customer: Description of the payment transaction. To display the description, set display_description to true in custom_elements., defaults to None
    :type customer: str, optional
    :param escrow: Determines whether the payment is held in escrow for later release., defaults to None
    :type escrow: bool, optional
    :param escrow_release_days: Determines the number of days after creation of the payment that funds are released from escrow. Funds are released at 5:00 pm GMT on the day indicated. Integer, range: 1-90., defaults to None
    :type escrow_release_days: float, optional
    :param id_: ID of the Rapyd checkout page. String starting with **checkout_**., defaults to None
    :type id_: str, optional
    :param merchant_main_button: A string that represents the text on the main Call to Action (CTA) button. One of the following:\<BR\>* place_your_order - Place Your Order.\<BR\>* pay - Pay.\<BR\>* pay_now - Pay Now.\<BR\>* make_payment - Make Payment.\<BR\>* purchase - Purchase.\<BR\>* buy - Buy.\<BR\>* donate - Donate.\<BR\>* To configure this button, use the Client Portal., defaults to None
    :type merchant_main_button: str, optional
    :param custom_elements: Description of the payment transaction., defaults to None
    :type custom_elements: HostedPageAdditionalResponseCustomElements, optional
    :param payment: Collects money from a payment method and deposits it into one or more Rapyd Wallets, defaults to None
    :type payment: Payment, optional
    :param payment_expiration: Length of time for the payment to be completed after it is created, measured in seconds. When both expiration and payment_expiration are set, the payment expires at the earlier time., defaults to None
    :type payment_expiration: float, optional
    :param payment_method_type: The type of the payment method. For example, **it_visa_card**, defaults to None
    :type payment_method_type: str, optional
    :param payment_method_type_categories: A list of the categories of payment method that are supported on the checkout page. The categories appear on the page in the order provided, defaults to None
    :type payment_method_type_categories: List[str], optional
    :param payment_method_types_exclude: List of payment methods that are excluded from display on the checkout page., defaults to None
    :type payment_method_types_exclude: List[str], optional
    :param payment_method_types_include: List of payment methods that are displayed on the checkout page. The payment methods appear on the page in the order provided., defaults to None
    :type payment_method_types_include: List[str], optional
    :param timestamp: Time of creation of the checkout page, in Unix time., defaults to None
    :type timestamp: float, optional
    """

    def __init__(
        self,
        language: str = None,
        merchant_alias: str = None,
        merchant_color: str = None,
        merchant_customer_support: MerchantCustomerSupport = None,
        merchant_logo: str = None,
        merchant_website: str = None,
        redirect_url: str = None,
        merchant_terms: str = None,
        cancel_url: str = None,
        complete_url: str = None,
        merchant_privacy_policy: str = None,
        page_expiration: float = None,
        status: HostedPageStatus = None,
        account_funding_transaction: dict = None,
        amount: float = None,
        cancel_checkout_url: str = None,
        cart_items: HostedPageAdditionalResponseCartItems = None,
        complete_checkout_url: str = None,
        country: str = None,
        currency: str = None,
        customer: str = None,
        escrow: bool = None,
        escrow_release_days: float = None,
        id_: str = None,
        merchant_main_button: str = None,
        custom_elements: HostedPageAdditionalResponseCustomElements = None,
        payment: Payment = None,
        payment_expiration: float = None,
        payment_method_type: str = None,
        payment_method_type_categories: List[str] = None,
        payment_method_types_exclude: List[str] = None,
        payment_method_types_include: List[str] = None,
        timestamp: float = None,
    ):
        """CheckoutPageResponse

        :param language: Determines the default language of the hosted page. When this parameter is null, the language of the user's browser is used. If the language of the user's browser cannot be determined, the default language is English, defaults to None
        :type language: str, optional
        :param merchant_alias: Reserved, the default name is Rapyd, defaults to None
        :type merchant_alias: str, optional
        :param merchant_color: Color of the call-to-action (CTA) button on the hosted page. To configure this field, use the Client Portal, defaults to None
        :type merchant_color: str, optional
        :param merchant_customer_support: Contains details of the client’s customer support. To configure these fields, use the Client Portal., defaults to None
        :type merchant_customer_support: MerchantCustomerSupport, optional
        :param merchant_logo: URL for the image of the client's logo. To configure this field, use the Client Portal, defaults to None
        :type merchant_logo: str, optional
        :param merchant_website: The URL where the customer is redirected after exiting the hosted page. Relevant when one or both of the following fields is unset: \<BR\>* `cancel_url` \<BR\>* `complete_url`.\<BR\> To configure this field, use the Client Portal, defaults to None
        :type merchant_website: str, optional
        :param redirect_url: URL of the hosted page that is shown to the customer., defaults to None
        :type redirect_url: str, optional
        :param merchant_terms: URL for the client's terms and conditions. To configure this field, use the Client Portal, defaults to None
        :type merchant_terms: str, optional
        :param cancel_url: URL where the customer is redirected after pressing Back to Website to exit the hosted page. This URL overrides the merchant_website URL. Does not support localhost URLs., defaults to None
        :type cancel_url: str, optional
        :param complete_url: URL where the customer is redirected after pressing Close to exit the hosted page. This URL overrides the merchant_website URL. Does not support localhost URLs., defaults to None
        :type complete_url: str, optional
        :param merchant_privacy_policy: URL for the terms and conditions of the agreement between the client and the client’s customers. To configure this field, use the Client Portal., defaults to None
        :type merchant_privacy_policy: str, optional
        :param page_expiration: Length of time for the payment to be completed after it is created, measured in seconds. When both expiration and payment_expiration are set, the payment expires at the earlier time., defaults to None
        :type page_expiration: float, optional
        :param status: Status of the hosted page. One of the following: NEW - The hosted page was created. DON - Done. The card was added to the customer profile. EXP - The hosted page expired. , defaults to None
        :type status: HostedPageStatus, optional
        :param account_funding_transaction: Details of an account funding transaction (AFT), which transfers funds from a card to a cardholder's wallet., defaults to None
        :type account_funding_transaction: dict, optional
        :param amount: The amount of the payment, in units of the currency defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015. If the amount is a whole number, use an integer and not a decimal., defaults to None
        :type amount: float, optional
        :param cancel_checkout_url: URL where the customer is redirected after pressing Back to Website to exit the hosted page. This URL overrides the merchant_website URL. Does not support localhost URLs., defaults to None
        :type cancel_checkout_url: str, optional
        :param cart_items: Describes the cart items that the customer is purchasing. These items are displayed at the checkout page., defaults to None
        :type cart_items: HostedPageAdditionalResponseCartItems, optional
        :param complete_checkout_url: URL where the customer is redirected after pressing Finish to exit the hosted page. This URL overrides the merchant_website URL. Does not support localhost URLs., defaults to None
        :type complete_checkout_url: str, optional
        :param country: country, defaults to None
        :type country: str, optional
        :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
        :type currency: str, optional
        :param customer: Description of the payment transaction. To display the description, set display_description to true in custom_elements., defaults to None
        :type customer: str, optional
        :param escrow: Determines whether the payment is held in escrow for later release., defaults to None
        :type escrow: bool, optional
        :param escrow_release_days: Determines the number of days after creation of the payment that funds are released from escrow. Funds are released at 5:00 pm GMT on the day indicated. Integer, range: 1-90., defaults to None
        :type escrow_release_days: float, optional
        :param id_: ID of the Rapyd checkout page. String starting with **checkout_**., defaults to None
        :type id_: str, optional
        :param merchant_main_button: A string that represents the text on the main Call to Action (CTA) button. One of the following:\<BR\>* place_your_order - Place Your Order.\<BR\>* pay - Pay.\<BR\>* pay_now - Pay Now.\<BR\>* make_payment - Make Payment.\<BR\>* purchase - Purchase.\<BR\>* buy - Buy.\<BR\>* donate - Donate.\<BR\>* To configure this button, use the Client Portal., defaults to None
        :type merchant_main_button: str, optional
        :param custom_elements: Description of the payment transaction., defaults to None
        :type custom_elements: HostedPageAdditionalResponseCustomElements, optional
        :param payment: Collects money from a payment method and deposits it into one or more Rapyd Wallets, defaults to None
        :type payment: Payment, optional
        :param payment_expiration: Length of time for the payment to be completed after it is created, measured in seconds. When both expiration and payment_expiration are set, the payment expires at the earlier time., defaults to None
        :type payment_expiration: float, optional
        :param payment_method_type: The type of the payment method. For example, **it_visa_card**, defaults to None
        :type payment_method_type: str, optional
        :param payment_method_type_categories: A list of the categories of payment method that are supported on the checkout page. The categories appear on the page in the order provided, defaults to None
        :type payment_method_type_categories: List[str], optional
        :param payment_method_types_exclude: List of payment methods that are excluded from display on the checkout page., defaults to None
        :type payment_method_types_exclude: List[str], optional
        :param payment_method_types_include: List of payment methods that are displayed on the checkout page. The payment methods appear on the page in the order provided., defaults to None
        :type payment_method_types_include: List[str], optional
        :param timestamp: Time of creation of the checkout page, in Unix time., defaults to None
        :type timestamp: float, optional
        """
        self.language = self._define_str("language", language, nullable=True)
        self.merchant_alias = self._define_str(
            "merchant_alias", merchant_alias, nullable=True
        )
        self.merchant_color = self._define_str(
            "merchant_color", merchant_color, nullable=True
        )
        self.merchant_customer_support = self._define_object(
            merchant_customer_support, MerchantCustomerSupport
        )
        self.merchant_logo = self._define_str(
            "merchant_logo", merchant_logo, nullable=True
        )
        self.merchant_website = self._define_str(
            "merchant_website", merchant_website, nullable=True
        )
        self.redirect_url = self._define_str(
            "redirect_url", redirect_url, nullable=True
        )
        self.merchant_terms = self._define_str(
            "merchant_terms", merchant_terms, nullable=True
        )
        self.cancel_url = self._define_str("cancel_url", cancel_url, nullable=True)
        self.complete_url = self._define_str(
            "complete_url", complete_url, nullable=True
        )
        self.merchant_privacy_policy = self._define_str(
            "merchant_privacy_policy", merchant_privacy_policy, nullable=True
        )
        self.page_expiration = self._define_number(
            "page_expiration", page_expiration, nullable=True
        )
        self.status = (
            self._enum_matching(status, HostedPageStatus.list(), "status")
            if status
            else None
        )
        self.account_funding_transaction = account_funding_transaction
        self.amount = self._define_number("amount", amount, nullable=True)
        self.cancel_checkout_url = self._define_str(
            "cancel_checkout_url", cancel_checkout_url, nullable=True
        )
        self.cart_items = self._define_object(
            cart_items, HostedPageAdditionalResponseCartItems
        )
        self.complete_checkout_url = self._define_str(
            "complete_checkout_url", complete_checkout_url, nullable=True
        )
        self.country = self._define_str(
            "country",
            country,
            nullable=True,
            pattern="Name of the country. Two-letter ISO 3166-1 alpha-2 code.",
        )
        self.currency = self._define_str(
            "currency",
            currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.customer = self._define_str("customer", customer, nullable=True)
        self.escrow = escrow
        self.escrow_release_days = self._define_number(
            "escrow_release_days", escrow_release_days, nullable=True
        )
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.merchant_main_button = self._define_str(
            "merchant_main_button", merchant_main_button, nullable=True
        )
        self.custom_elements = self._define_object(
            custom_elements, HostedPageAdditionalResponseCustomElements
        )
        self.payment = self._define_object(payment, Payment)
        self.payment_expiration = self._define_number(
            "payment_expiration", payment_expiration, nullable=True
        )
        self.payment_method_type = self._define_str(
            "payment_method_type", payment_method_type, nullable=True
        )
        self.payment_method_type_categories = payment_method_type_categories
        self.payment_method_types_exclude = payment_method_types_exclude
        self.payment_method_types_include = payment_method_types_include
        self.timestamp = self._define_number("timestamp", timestamp, nullable=True)



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .list_webhooks_response_attempts import ListWebhooksResponseAttempts


@JsonMap({"type_": "type"})
class ListWebhooksResponse(BaseModel):
    """ListWebhooksResponse

    :param attempts: List of attempts at sending the webhook. Array of objects that contain the following fields:, defaults to None
    :type attempts: ListWebhooksResponseAttempts, optional
    :param created_at: Timestamp for the creation of the webhook. Unix time., defaults to None
    :type created_at: float, optional
    :param data: The data section of the webhook itself., defaults to None
    :type data: dict, optional
    :param last_attempt_at: Timestamp of the last attempt at sending the webhook. Unix time., defaults to None
    :type last_attempt_at: float, optional
    :param next_attempt_at: Timestamp of the next attempt at sending the webhook. Unix time., defaults to None
    :type next_attempt_at: float, optional
    :param status: Status of the webhook. One of the following: \<BR\> * **NEW** (new) - The webhook was created and has not yet been sent successfully. \<BR\> * **CLO** (closed) - The webhook was sent successfully.\<BR\> * **ERR** (error) - Attempts were made to send the webhook, but the maximum number of retries was reached. The automatic retry process failed. The webhook was not sent. \<BR\> * **RET** (retried) - The webhook was resent., defaults to None
    :type status: str, optional
    :param token: ID of the webhook. String starting with **wh_**., defaults to None
    :type token: str, optional
    :param type_: Internal name of the webhook type., defaults to None
    :type type_: float, optional
    """

    def __init__(
        self,
        attempts: ListWebhooksResponseAttempts = None,
        created_at: float = None,
        data: dict = None,
        last_attempt_at: float = None,
        next_attempt_at: float = None,
        status: str = None,
        token: str = None,
        type_: float = None,
    ):
        """ListWebhooksResponse

        :param attempts: List of attempts at sending the webhook. Array of objects that contain the following fields:, defaults to None
        :type attempts: ListWebhooksResponseAttempts, optional
        :param created_at: Timestamp for the creation of the webhook. Unix time., defaults to None
        :type created_at: float, optional
        :param data: The data section of the webhook itself., defaults to None
        :type data: dict, optional
        :param last_attempt_at: Timestamp of the last attempt at sending the webhook. Unix time., defaults to None
        :type last_attempt_at: float, optional
        :param next_attempt_at: Timestamp of the next attempt at sending the webhook. Unix time., defaults to None
        :type next_attempt_at: float, optional
        :param status: Status of the webhook. One of the following: \<BR\> * **NEW** (new) - The webhook was created and has not yet been sent successfully. \<BR\> * **CLO** (closed) - The webhook was sent successfully.\<BR\> * **ERR** (error) - Attempts were made to send the webhook, but the maximum number of retries was reached. The automatic retry process failed. The webhook was not sent. \<BR\> * **RET** (retried) - The webhook was resent., defaults to None
        :type status: str, optional
        :param token: ID of the webhook. String starting with **wh_**., defaults to None
        :type token: str, optional
        :param type_: Internal name of the webhook type., defaults to None
        :type type_: float, optional
        """
        self.attempts = self._define_object(attempts, ListWebhooksResponseAttempts)
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.data = data
        self.last_attempt_at = self._define_number(
            "last_attempt_at", last_attempt_at, nullable=True
        )
        self.next_attempt_at = self._define_number(
            "next_attempt_at", next_attempt_at, nullable=True
        )
        self.status = self._define_str("status", status, nullable=True)
        self.token = self._define_str("token", token, nullable=True)
        self.type_ = self._define_number("type_", type_, nullable=True)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class PayoutAmountRangePerCurrencyInner(BaseModel):
    """PayoutAmountRangePerCurrencyInner

    :param maximum_amount: Maximum amount supported by this payout method for the indicated currency. Decimal number., defaults to None
    :type maximum_amount: float, optional
    :param minimum_amount: Minimum amount supported by this payout method for the indicated currency. Decimal number., defaults to None
    :type minimum_amount: float, optional
    :param payout_currency: payout_currency, defaults to None
    :type payout_currency: str, optional
    """

    def __init__(
        self,
        maximum_amount: float = None,
        minimum_amount: float = None,
        payout_currency: str = None,
    ):
        """PayoutAmountRangePerCurrencyInner

        :param maximum_amount: Maximum amount supported by this payout method for the indicated currency. Decimal number., defaults to None
        :type maximum_amount: float, optional
        :param minimum_amount: Minimum amount supported by this payout method for the indicated currency. Decimal number., defaults to None
        :type minimum_amount: float, optional
        :param payout_currency: payout_currency, defaults to None
        :type payout_currency: str, optional
        """
        self.maximum_amount = self._define_number(
            "maximum_amount", maximum_amount, nullable=True
        )
        self.minimum_amount = self._define_number(
            "minimum_amount", minimum_amount, nullable=True
        )
        self.payout_currency = self._define_str(
            "payout_currency",
            payout_currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .entity_type import EntityType


@JsonMap({"id_": "id"})
class Sender(BaseModel):
    """Sender

    :param account_number: Bank account number of sender., defaults to None
    :type account_number: str, optional
    :param additional_data: Additional data required by the payout method. Object.., defaults to None
    :type additional_data: dict, optional
    :param additional_last_name: The person's second last name., defaults to None
    :type additional_last_name: str, optional
    :param address: The address of the sender., defaults to None
    :type address: str, optional
    :param bank_code: Domestic identifier of the bank., defaults to None
    :type bank_code: str, optional
    :param bank_name: Name of the bank., defaults to None
    :type bank_name: str, optional
    :param beneficiary_relationship: When the beneficiary is not a relative of the sender, the relationship should be specified as client or non-relative., defaults to None
    :type beneficiary_relationship: str, optional
    :param city: City where the sender is located., defaults to None
    :type city: str, optional
    :param company_name: Name of the sender company. Relevant when entity_type is company., defaults to None
    :type company_name: str, optional
    :param country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
    :type country: str, optional
    :param currency: currency, defaults to None
    :type currency: str, optional
    :param date_of_birth: The date of birth of the sender. Relevant when entity_type is individual., defaults to None
    :type date_of_birth: str, optional
    :param date_of_incorporation: Date of incorporation of the corporation., defaults to None
    :type date_of_incorporation: str, optional
    :param email: The person's email address., defaults to None
    :type email: str, optional
    :param entity_type: entity_type, defaults to None
    :type entity_type: EntityType, optional
    :param first_name: First name of the sender. Relevant when `entity_type` is **individual**., defaults to None
    :type first_name: str, optional
    :param id_: ID of the Sender object. String starting with **sender_**. Response only., defaults to None
    :type id_: str, optional
    :param id_date_of_issue: The date of issuance of the person's identification document., defaults to None
    :type id_date_of_issue: str, optional
    :param id_expiry: The expiration date of the person's identification document., defaults to None
    :type id_expiry: str, optional
    :param id_issue_authority: The government agency that issued the person's identification document., defaults to None
    :type id_issue_authority: str, optional
    :param id_issue_location: The location where the person's identification document was issued., defaults to None
    :type id_issue_location: str, optional
    :param identification_type: Type of identification document for the sender. When entity_type is company, this field must be company_registered_number. When entity_type is individual, one of the following values:  * drivers_license  * identification_id  * international_passport  * residence_permit* social_security  * work_permit , defaults to None
    :type identification_type: str, optional
    :param identification_value: identification number on the document mentioned in `identification_type`., defaults to None
    :type identification_value: str, optional
    :param last_name: Last name of the sender. Relevant when `entity_type` is individual., defaults to None
    :type last_name: str, optional
    :param merchant_reference_id: ID defined by the merchant. Rapyd does not validate this value to enforce uniqueness., defaults to None
    :type merchant_reference_id: str, optional
    :param middle_name: The person's middle name., defaults to None
    :type middle_name: str, optional
    :param mobile_number: Mobile phone number in E.164 format., defaults to None
    :type mobile_number: str, optional
    :param name: Name of the sender. Relevant when `entity_type` is company., defaults to None
    :type name: str, optional
    :param nationality: The person's nationality., defaults to None
    :type nationality: str, optional
    :param occupation: The person's occupation., defaults to None
    :type occupation: str, optional
    :param phone_country_code: The country code of the person's phone., defaults to None
    :type phone_country_code: str, optional
    :param phone_number: Phone number in E.164 format., defaults to None
    :type phone_number: str, optional
    :param postcode: Postal code of the sender., defaults to None
    :type postcode: str, optional
    :param province: Province portion of the address., defaults to None
    :type province: str, optional
    :param purpose_code: Reason for the payout., defaults to None
    :type purpose_code: str, optional
    :param source_of_income: The person's source of income., defaults to None
    :type source_of_income: str, optional
    :param state: State where the sender is located., defaults to None
    :type state: str, optional
    :param suburb: District of the sender's or beneficiary's city., defaults to None
    :type suburb: str, optional
    :param tax_id: Tax identification number., defaults to None
    :type tax_id: str, optional
    """

    def __init__(
        self,
        account_number: str = None,
        additional_data: dict = None,
        additional_last_name: str = None,
        address: str = None,
        bank_code: str = None,
        bank_name: str = None,
        beneficiary_relationship: str = None,
        city: str = None,
        company_name: str = None,
        country: str = None,
        currency: str = None,
        date_of_birth: str = None,
        date_of_incorporation: str = None,
        email: str = None,
        entity_type: EntityType = None,
        first_name: str = None,
        id_: str = None,
        id_date_of_issue: str = None,
        id_expiry: str = None,
        id_issue_authority: str = None,
        id_issue_location: str = None,
        identification_type: str = None,
        identification_value: str = None,
        last_name: str = None,
        merchant_reference_id: str = None,
        middle_name: str = None,
        mobile_number: str = None,
        name: str = None,
        nationality: str = None,
        occupation: str = None,
        phone_country_code: str = None,
        phone_number: str = None,
        postcode: str = None,
        province: str = None,
        purpose_code: str = None,
        source_of_income: str = None,
        state: str = None,
        suburb: str = None,
        tax_id: str = None,
    ):
        """Sender

        :param account_number: Bank account number of sender., defaults to None
        :type account_number: str, optional
        :param additional_data: Additional data required by the payout method. Object.., defaults to None
        :type additional_data: dict, optional
        :param additional_last_name: The person's second last name., defaults to None
        :type additional_last_name: str, optional
        :param address: The address of the sender., defaults to None
        :type address: str, optional
        :param bank_code: Domestic identifier of the bank., defaults to None
        :type bank_code: str, optional
        :param bank_name: Name of the bank., defaults to None
        :type bank_name: str, optional
        :param beneficiary_relationship: When the beneficiary is not a relative of the sender, the relationship should be specified as client or non-relative., defaults to None
        :type beneficiary_relationship: str, optional
        :param city: City where the sender is located., defaults to None
        :type city: str, optional
        :param company_name: Name of the sender company. Relevant when entity_type is company., defaults to None
        :type company_name: str, optional
        :param country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
        :type country: str, optional
        :param currency: currency, defaults to None
        :type currency: str, optional
        :param date_of_birth: The date of birth of the sender. Relevant when entity_type is individual., defaults to None
        :type date_of_birth: str, optional
        :param date_of_incorporation: Date of incorporation of the corporation., defaults to None
        :type date_of_incorporation: str, optional
        :param email: The person's email address., defaults to None
        :type email: str, optional
        :param entity_type: entity_type, defaults to None
        :type entity_type: EntityType, optional
        :param first_name: First name of the sender. Relevant when `entity_type` is **individual**., defaults to None
        :type first_name: str, optional
        :param id_: ID of the Sender object. String starting with **sender_**. Response only., defaults to None
        :type id_: str, optional
        :param id_date_of_issue: The date of issuance of the person's identification document., defaults to None
        :type id_date_of_issue: str, optional
        :param id_expiry: The expiration date of the person's identification document., defaults to None
        :type id_expiry: str, optional
        :param id_issue_authority: The government agency that issued the person's identification document., defaults to None
        :type id_issue_authority: str, optional
        :param id_issue_location: The location where the person's identification document was issued., defaults to None
        :type id_issue_location: str, optional
        :param identification_type: Type of identification document for the sender. When entity_type is company, this field must be company_registered_number. When entity_type is individual, one of the following values:  * drivers_license  * identification_id  * international_passport  * residence_permit* social_security  * work_permit , defaults to None
        :type identification_type: str, optional
        :param identification_value: identification number on the document mentioned in `identification_type`., defaults to None
        :type identification_value: str, optional
        :param last_name: Last name of the sender. Relevant when `entity_type` is individual., defaults to None
        :type last_name: str, optional
        :param merchant_reference_id: ID defined by the merchant. Rapyd does not validate this value to enforce uniqueness., defaults to None
        :type merchant_reference_id: str, optional
        :param middle_name: The person's middle name., defaults to None
        :type middle_name: str, optional
        :param mobile_number: Mobile phone number in E.164 format., defaults to None
        :type mobile_number: str, optional
        :param name: Name of the sender. Relevant when `entity_type` is company., defaults to None
        :type name: str, optional
        :param nationality: The person's nationality., defaults to None
        :type nationality: str, optional
        :param occupation: The person's occupation., defaults to None
        :type occupation: str, optional
        :param phone_country_code: The country code of the person's phone., defaults to None
        :type phone_country_code: str, optional
        :param phone_number: Phone number in E.164 format., defaults to None
        :type phone_number: str, optional
        :param postcode: Postal code of the sender., defaults to None
        :type postcode: str, optional
        :param province: Province portion of the address., defaults to None
        :type province: str, optional
        :param purpose_code: Reason for the payout., defaults to None
        :type purpose_code: str, optional
        :param source_of_income: The person's source of income., defaults to None
        :type source_of_income: str, optional
        :param state: State where the sender is located., defaults to None
        :type state: str, optional
        :param suburb: District of the sender's or beneficiary's city., defaults to None
        :type suburb: str, optional
        :param tax_id: Tax identification number., defaults to None
        :type tax_id: str, optional
        """
        self.account_number = self._define_str(
            "account_number", account_number, nullable=True
        )
        self.additional_data = additional_data
        self.additional_last_name = self._define_str(
            "additional_last_name", additional_last_name, nullable=True
        )
        self.address = self._define_str("address", address, nullable=True)
        self.bank_code = self._define_str("bank_code", bank_code, nullable=True)
        self.bank_name = self._define_str("bank_name", bank_name, nullable=True)
        self.beneficiary_relationship = self._define_str(
            "beneficiary_relationship", beneficiary_relationship, nullable=True
        )
        self.city = self._define_str("city", city, nullable=True)
        self.company_name = self._define_str(
            "company_name", company_name, nullable=True
        )
        self.country = self._define_str("country", country, nullable=True)
        self.currency = self._define_str(
            "currency",
            currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.date_of_birth = self._define_str(
            "date_of_birth", date_of_birth, nullable=True
        )
        self.date_of_incorporation = self._define_str(
            "date_of_incorporation", date_of_incorporation, nullable=True
        )
        self.email = self._define_str("email", email, nullable=True)
        self.entity_type = (
            self._enum_matching(entity_type, EntityType.list(), "entity_type")
            if entity_type
            else None
        )
        self.first_name = self._define_str("first_name", first_name, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.id_date_of_issue = self._define_str(
            "id_date_of_issue", id_date_of_issue, nullable=True
        )
        self.id_expiry = self._define_str("id_expiry", id_expiry, nullable=True)
        self.id_issue_authority = self._define_str(
            "id_issue_authority", id_issue_authority, nullable=True
        )
        self.id_issue_location = self._define_str(
            "id_issue_location", id_issue_location, nullable=True
        )
        self.identification_type = self._define_str(
            "identification_type", identification_type, nullable=True
        )
        self.identification_value = self._define_str(
            "identification_value", identification_value, nullable=True
        )
        self.last_name = self._define_str("last_name", last_name, nullable=True)
        self.merchant_reference_id = self._define_str(
            "merchant_reference_id", merchant_reference_id, nullable=True
        )
        self.middle_name = self._define_str("middle_name", middle_name, nullable=True)
        self.mobile_number = self._define_str(
            "mobile_number", mobile_number, nullable=True
        )
        self.name = self._define_str("name", name, nullable=True)
        self.nationality = self._define_str("nationality", nationality, nullable=True)
        self.occupation = self._define_str("occupation", occupation, nullable=True)
        self.phone_country_code = self._define_str(
            "phone_country_code", phone_country_code, nullable=True
        )
        self.phone_number = self._define_str(
            "phone_number", phone_number, nullable=True
        )
        self.postcode = self._define_str("postcode", postcode, nullable=True)
        self.province = self._define_str("province", province, nullable=True)
        self.purpose_code = self._define_str(
            "purpose_code", purpose_code, nullable=True
        )
        self.source_of_income = self._define_str(
            "source_of_income", source_of_income, nullable=True
        )
        self.state = self._define_str("state", state, nullable=True)
        self.suburb = self._define_str("suburb", suburb, nullable=True)
        self.tax_id = self._define_str("tax_id", tax_id, nullable=True)



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .entity_type import EntityType
from .payout_required_fields import PayoutRequiredFields


@JsonMap({})
class PayoutMethodTypeDetails(BaseModel):
    """PayoutMethodTypeDetails

    :param batch_file_header: concatenation of all fields., defaults to None
    :type batch_file_header: str, optional
    :param beneficiary_country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
    :type beneficiary_country: str, optional
    :param beneficiary_entity_type: beneficiary_entity_type, defaults to None
    :type beneficiary_entity_type: EntityType, optional
    :param beneficiary_required_fields: Lists all the beneficiary required fields for payout method, defaults to None
    :type beneficiary_required_fields: List[PayoutRequiredFields], optional
    :param image: URL of an image that the merchant can use to represent the payout method., defaults to None
    :type image: str, optional
    :param is_cancelable: Indicates whether the payout can be canceled. Relevant when category is cash. One of the following values 0 - Not cancelable. 1 - cancelable.', defaults to None
    :type is_cancelable: int, optional
    :param is_expirable: Indicates whether the payout expires if not completed. Relevant when category is cash. One of the following values 0 - Not expirable. 1 - expirable.', defaults to None
    :type is_expirable: int, optional
    :param is_location_specific: Indicates whether the payout must be made at a specific physical location. Relevant when category is cash. One of the following values 0 - Not location specific. 1 - location specific.', defaults to None
    :type is_location_specific: int, optional
    :param is_online: Indicates whether the payout is made automatically, without any action by the beneficiary., defaults to None
    :type is_online: bool, optional
    :param maximum_amount: Maximum amount supported by this payout method for the indicated currency. Decimal number., defaults to None
    :type maximum_amount: float, optional
    :param maximum_expiration_seconds: The maximum time (in seconds) that the merchant can set for completing the payout. Relevant when is_expirable is true., defaults to None
    :type maximum_expiration_seconds: float, optional
    :param minimum_amount: Minimum amount supported by this payout method for the indicated currency. Decimal number., defaults to None
    :type minimum_amount: float, optional
    :param minimum_expiration_seconds: The minimum time (in seconds) that the merchant can set for completing the payout. Relevant when is_expirable is true., defaults to None
    :type minimum_expiration_seconds: float, optional
    :param payout_currency: payout_currency, defaults to None
    :type payout_currency: str, optional
    :param payout_method_type: The type of payout method. The two-letter prefix must match the beneficiary country code. Required when default_payout_method_type is not used., defaults to None
    :type payout_method_type: str, optional
    :param payout_options: Lists all the payout options for payout method., defaults to None
    :type payout_options: List[PayoutRequiredFields], optional
    :param sender_country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
    :type sender_country: str, optional
    :param sender_currency: sender_currency, defaults to None
    :type sender_currency: str, optional
    :param sender_entity_type: sender_entity_type, defaults to None
    :type sender_entity_type: EntityType, optional
    :param sender_required_fields: Lists all the sender required fields for payout method, defaults to None
    :type sender_required_fields: List[PayoutRequiredFields], optional
    :param status: Indicates whether the payout method is currently available. One of the following values: 0 - Not available. 1 - Available., defaults to None
    :type status: int, optional
    """

    def __init__(
        self,
        batch_file_header: str = None,
        beneficiary_country: str = None,
        beneficiary_entity_type: EntityType = None,
        beneficiary_required_fields: List[PayoutRequiredFields] = None,
        image: str = None,
        is_cancelable: int = None,
        is_expirable: int = None,
        is_location_specific: int = None,
        is_online: bool = None,
        maximum_amount: float = None,
        maximum_expiration_seconds: float = None,
        minimum_amount: float = None,
        minimum_expiration_seconds: float = None,
        payout_currency: str = None,
        payout_method_type: str = None,
        payout_options: List[PayoutRequiredFields] = None,
        sender_country: str = None,
        sender_currency: str = None,
        sender_entity_type: EntityType = None,
        sender_required_fields: List[PayoutRequiredFields] = None,
        status: int = None,
    ):
        """PayoutMethodTypeDetails

        :param batch_file_header: concatenation of all fields., defaults to None
        :type batch_file_header: str, optional
        :param beneficiary_country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
        :type beneficiary_country: str, optional
        :param beneficiary_entity_type: beneficiary_entity_type, defaults to None
        :type beneficiary_entity_type: EntityType, optional
        :param beneficiary_required_fields: Lists all the beneficiary required fields for payout method, defaults to None
        :type beneficiary_required_fields: List[PayoutRequiredFields], optional
        :param image: URL of an image that the merchant can use to represent the payout method., defaults to None
        :type image: str, optional
        :param is_cancelable: Indicates whether the payout can be canceled. Relevant when category is cash. One of the following values 0 - Not cancelable. 1 - cancelable.', defaults to None
        :type is_cancelable: int, optional
        :param is_expirable: Indicates whether the payout expires if not completed. Relevant when category is cash. One of the following values 0 - Not expirable. 1 - expirable.', defaults to None
        :type is_expirable: int, optional
        :param is_location_specific: Indicates whether the payout must be made at a specific physical location. Relevant when category is cash. One of the following values 0 - Not location specific. 1 - location specific.', defaults to None
        :type is_location_specific: int, optional
        :param is_online: Indicates whether the payout is made automatically, without any action by the beneficiary., defaults to None
        :type is_online: bool, optional
        :param maximum_amount: Maximum amount supported by this payout method for the indicated currency. Decimal number., defaults to None
        :type maximum_amount: float, optional
        :param maximum_expiration_seconds: The maximum time (in seconds) that the merchant can set for completing the payout. Relevant when is_expirable is true., defaults to None
        :type maximum_expiration_seconds: float, optional
        :param minimum_amount: Minimum amount supported by this payout method for the indicated currency. Decimal number., defaults to None
        :type minimum_amount: float, optional
        :param minimum_expiration_seconds: The minimum time (in seconds) that the merchant can set for completing the payout. Relevant when is_expirable is true., defaults to None
        :type minimum_expiration_seconds: float, optional
        :param payout_currency: payout_currency, defaults to None
        :type payout_currency: str, optional
        :param payout_method_type: The type of payout method. The two-letter prefix must match the beneficiary country code. Required when default_payout_method_type is not used., defaults to None
        :type payout_method_type: str, optional
        :param payout_options: Lists all the payout options for payout method., defaults to None
        :type payout_options: List[PayoutRequiredFields], optional
        :param sender_country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
        :type sender_country: str, optional
        :param sender_currency: sender_currency, defaults to None
        :type sender_currency: str, optional
        :param sender_entity_type: sender_entity_type, defaults to None
        :type sender_entity_type: EntityType, optional
        :param sender_required_fields: Lists all the sender required fields for payout method, defaults to None
        :type sender_required_fields: List[PayoutRequiredFields], optional
        :param status: Indicates whether the payout method is currently available. One of the following values: 0 - Not available. 1 - Available., defaults to None
        :type status: int, optional
        """
        self.batch_file_header = self._define_str(
            "batch_file_header", batch_file_header, nullable=True
        )
        self.beneficiary_country = self._define_str(
            "beneficiary_country", beneficiary_country, nullable=True
        )
        self.beneficiary_entity_type = (
            self._enum_matching(
                beneficiary_entity_type, EntityType.list(), "beneficiary_entity_type"
            )
            if beneficiary_entity_type
            else None
        )
        self.beneficiary_required_fields = self._define_list(
            beneficiary_required_fields, PayoutRequiredFields
        )
        self.image = self._define_str("image", image, nullable=True)
        self.is_cancelable = self._define_number(
            "is_cancelable", is_cancelable, nullable=True
        )
        self.is_expirable = self._define_number(
            "is_expirable", is_expirable, nullable=True
        )
        self.is_location_specific = self._define_number(
            "is_location_specific", is_location_specific, nullable=True
        )
        self.is_online = is_online
        self.maximum_amount = self._define_number(
            "maximum_amount", maximum_amount, nullable=True
        )
        self.maximum_expiration_seconds = self._define_number(
            "maximum_expiration_seconds", maximum_expiration_seconds, nullable=True
        )
        self.minimum_amount = self._define_number(
            "minimum_amount", minimum_amount, nullable=True
        )
        self.minimum_expiration_seconds = self._define_number(
            "minimum_expiration_seconds", minimum_expiration_seconds, nullable=True
        )
        self.payout_currency = self._define_str(
            "payout_currency",
            payout_currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.payout_method_type = self._define_str(
            "payout_method_type", payout_method_type, nullable=True
        )
        self.payout_options = self._define_list(payout_options, PayoutRequiredFields)
        self.sender_country = self._define_str(
            "sender_country", sender_country, nullable=True
        )
        self.sender_currency = self._define_str(
            "sender_currency",
            sender_currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.sender_entity_type = (
            self._enum_matching(
                sender_entity_type, EntityType.list(), "sender_entity_type"
            )
            if sender_entity_type
            else None
        )
        self.sender_required_fields = self._define_list(
            sender_required_fields, PayoutRequiredFields
        )
        self.status = self._define_number("status", status, nullable=True)



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .status_1 import Status1
from .discount import Discount


@JsonMap({})
class InlineResponse200_20(BaseModel):
    """InlineResponse200_20

    :param status: status, defaults to None
    :type status: Status1, optional
    :param data: Describes the fields relating to discounts in REST messages and webhooks for customer profiles and subscriptions Contains information about the coupon that applies to the customer. Read-only field. Adding a discount is a 2-step process: \<BR\> 1. Create Coupon, which returns a coupon ID. \<BR\>2. Add the coupon ID to the coupon field of the customer with Create Customer or Update Customer., defaults to None
    :type data: Discount, optional
    """

    def __init__(self, status: Status1 = None, data: Discount = None):
        """InlineResponse200_20

        :param status: status, defaults to None
        :type status: Status1, optional
        :param data: Describes the fields relating to discounts in REST messages and webhooks for customer profiles and subscriptions Contains information about the coupon that applies to the customer. Read-only field. Adding a discount is a 2-step process: \<BR\> 1. Create Coupon, which returns a coupon ID. \<BR\>2. Add the coupon ID to the coupon field of the customer with Create Customer or Update Customer., defaults to None
        :type data: Discount, optional
        """
        self.status = self._define_object(status, Status1)
        self.data = self._define_object(data, Discount)



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .order_returned_item_response import OrderReturnedItemResponse


@JsonMap({"id_": "id"})
class OrderReturnedResponse(BaseModel):
    """OrderReturnedResponse

    :param id_: id_, defaults to None
    :type id_: str, optional
    :param amount: amount, defaults to None
    :type amount: float, optional
    :param created: created, defaults to None
    :type created: int, optional
    :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
    :type currency: str, optional
    :param items: items, defaults to None
    :type items: List[OrderReturnedItemResponse], optional
    :param order: order, defaults to None
    :type order: str, optional
    :param refund: refund, defaults to None
    :type refund: str, optional
    """

    def __init__(
        self,
        id_: str = None,
        amount: float = None,
        created: int = None,
        currency: str = None,
        items: List[OrderReturnedItemResponse] = None,
        order: str = None,
        refund: str = None,
    ):
        """OrderReturnedResponse

        :param id_: id_, defaults to None
        :type id_: str, optional
        :param amount: amount, defaults to None
        :type amount: float, optional
        :param created: created, defaults to None
        :type created: int, optional
        :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
        :type currency: str, optional
        :param items: items, defaults to None
        :type items: List[OrderReturnedItemResponse], optional
        :param order: order, defaults to None
        :type order: str, optional
        :param refund: refund, defaults to None
        :type refund: str, optional
        """
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.amount = self._define_number("amount", amount, nullable=True)
        self.created = self._define_number("created", created, nullable=True)
        self.currency = self._define_str(
            "currency",
            currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.items = self._define_list(items, OrderReturnedItemResponse)
        self.order = self._define_str("order", order, nullable=True)
        self.refund = self._define_str("refund", refund, nullable=True)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class CardTokenResponseCardFields(BaseModel):
    """Contains details about the card.

    :param recurrence_type: Specifies the primary intended purpose of the saved payment method. See Saving a European Card While Creating a Payment. One of the following values: \<BR\> * **installment - Regular payments for a defined number of payment cycles.** \<BR\> * **recurring** - Regular payments for an indefinite period. \<BR\> * **unscheduled** - Individual unrelated payments., defaults to None
    :type recurrence_type: str, optional
    """

    def __init__(self, recurrence_type: str = None):
        """Contains details about the card.

        :param recurrence_type: Specifies the primary intended purpose of the saved payment method. See Saving a European Card While Creating a Payment. One of the following values: \<BR\> * **installment - Regular payments for a defined number of payment cycles.** \<BR\> * **recurring** - Regular payments for an indefinite period. \<BR\> * **unscheduled** - Individual unrelated payments., defaults to None
        :type recurrence_type: str, optional
        """
        self.recurrence_type = self._define_str(
            "recurrence_type", recurrence_type, nullable=True
        )



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .list_supported_languages_response_languages import (
    ListSupportedLanguagesResponseLanguages,
)


@JsonMap({})
class ListSupportedLanguagesResponse(BaseModel):
    """ListSupportedLanguagesResponse

    :param languages: List of the languages supported for hosted pages., defaults to None
    :type languages: ListSupportedLanguagesResponseLanguages, optional
    """

    def __init__(self, languages: ListSupportedLanguagesResponseLanguages = None):
        """ListSupportedLanguagesResponse

        :param languages: List of the languages supported for hosted pages., defaults to None
        :type languages: ListSupportedLanguagesResponseLanguages, optional
        """
        self.languages = self._define_object(
            languages, ListSupportedLanguagesResponseLanguages
        )



from __future__ import annotations
from typing import Union
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.one_of_base_model import OneOfBaseModel
from .entity_type import EntityType
from .beneficiary import Beneficiary


class BeneficiaryValidateBodyBeneficiaryGuard(OneOfBaseModel):
    class_list = {"str": str, "Beneficiary": Beneficiary}


BeneficiaryValidateBodyBeneficiary = Union[str, Beneficiary]


@JsonMap({})
class BeneficiaryValidateBody(BaseModel):
    """BeneficiaryValidateBody

    :param amount: Maximum payout amount to validate, in units of the currency defined in `sender_currency`. Decimal.
    :type amount: float
    :param beneficiary: String starting with **beneficiary_** or the object describing the beneficiary.
    :type beneficiary: BeneficiaryValidateBodyBeneficiary
    :param identifier_type: Identifier type of beneficiary, defaults to None
    :type identifier_type: str, optional
    :param identifier_value: Identifier value to filter with, defaults to None
    :type identifier_value: str, optional
    :param payout_method_type: The type of the payout method. Set to a value included in the response to List Payout Method Types. The two-letter prefix must match the beneficiary country code.
    :type payout_method_type: str
    :param sender_country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code.
    :type sender_country: str
    :param sender_currency: sender_currency
    :type sender_currency: str
    :param sender_entity_type: sender_entity_type
    :type sender_entity_type: EntityType
    """

    def __init__(
        self,
        amount: float,
        beneficiary: BeneficiaryValidateBodyBeneficiary,
        payout_method_type: str,
        sender_country: str,
        sender_currency: str,
        sender_entity_type: EntityType,
        identifier_type: str = None,
        identifier_value: str = None,
    ):
        """BeneficiaryValidateBody

        :param amount: Maximum payout amount to validate, in units of the currency defined in `sender_currency`. Decimal.
        :type amount: float
        :param beneficiary: String starting with **beneficiary_** or the object describing the beneficiary.
        :type beneficiary: BeneficiaryValidateBodyBeneficiary
        :param identifier_type: Identifier type of beneficiary, defaults to None
        :type identifier_type: str, optional
        :param identifier_value: Identifier value to filter with, defaults to None
        :type identifier_value: str, optional
        :param payout_method_type: The type of the payout method. Set to a value included in the response to List Payout Method Types. The two-letter prefix must match the beneficiary country code.
        :type payout_method_type: str
        :param sender_country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code.
        :type sender_country: str
        :param sender_currency: sender_currency
        :type sender_currency: str
        :param sender_entity_type: sender_entity_type
        :type sender_entity_type: EntityType
        """
        self.amount = amount
        self.beneficiary = BeneficiaryValidateBodyBeneficiaryGuard.return_one_of(
            beneficiary
        )
        self.identifier_type = self._define_str(
            "identifier_type", identifier_type, nullable=True
        )
        self.identifier_value = self._define_str(
            "identifier_value", identifier_value, nullable=True
        )
        self.payout_method_type = payout_method_type
        self.sender_country = sender_country
        self.sender_currency = self._define_str(
            "sender_currency",
            sender_currency,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.sender_entity_type = self._enum_matching(
            sender_entity_type, EntityType.list(), "sender_entity_type"
        )



from __future__ import annotations
from typing import Union
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.one_of_base_model import OneOfBaseModel
from .category import Category
from .next_action import NextAction
from .bin_details import BinDetails


@JsonMap({"id_": "id", "type_": "type"})
class CustomerPaymentMethod1(BaseModel):
    """CustomerPaymentMethod1

    :param bic_swift: BIC (SWIFT) code for the bank account., defaults to None
    :type bic_swift: str, optional
    :param account_last4: Last four digits of the bank account number or IBAN (International Bank Account Number)., defaults to None
    :type account_last4: str, optional
    :param id_: ID of the Payment Method object. String starting with **card_** or **other_**., defaults to None
    :type id_: str, optional
    :param type_: Name of the payment method type. For example, **it_visa_card**. To get a list of payment methods for a country, use 'List Payment Methods by Country'., defaults to None
    :type type_: str, optional
    :param category: category, defaults to None
    :type category: Category, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param image: A URL to the image of the icon for the type of payment method. Response only, defaults to None
    :type image: str, optional
    :param webhook_url: Reserved. Response only, defaults to None
    :type webhook_url: str, optional
    :param supporting_documentation: Reserved. Response only., defaults to None
    :type supporting_documentation: str, optional
    :param next_action: Indicates the next action for completing the payment. Response only. One of the following values are - * 3d_verification - The next action is 3DS authentication. To simulate 3DS authentication in the sandbox, see Simulating 3DS Authentication. Relevant only to card payments. * pending_capture - The next action is pending the capture of the amount. Relevant only to card payments when the amount is not zero. * pending_confirmation - The next action is pending the confirmation for the payment. Relevant to all payment methods excluding card payment. * not_applicable - The payment has completed or the next action is not relevant., defaults to None
    :type next_action: NextAction, optional
    """

    def __init__(
        self,
        bic_swift: str = None,
        account_last4: str = None,
        id_: str = None,
        type_: str = None,
        category: Category = None,
        metadata: dict = None,
        image: str = None,
        webhook_url: str = None,
        supporting_documentation: str = None,
        next_action: NextAction = None,
    ):
        """CustomerPaymentMethod1

        :param bic_swift: BIC (SWIFT) code for the bank account., defaults to None
        :type bic_swift: str, optional
        :param account_last4: Last four digits of the bank account number or IBAN (International Bank Account Number)., defaults to None
        :type account_last4: str, optional
        :param id_: ID of the Payment Method object. String starting with **card_** or **other_**., defaults to None
        :type id_: str, optional
        :param type_: Name of the payment method type. For example, **it_visa_card**. To get a list of payment methods for a country, use 'List Payment Methods by Country'., defaults to None
        :type type_: str, optional
        :param category: category, defaults to None
        :type category: Category, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param image: A URL to the image of the icon for the type of payment method. Response only, defaults to None
        :type image: str, optional
        :param webhook_url: Reserved. Response only, defaults to None
        :type webhook_url: str, optional
        :param supporting_documentation: Reserved. Response only., defaults to None
        :type supporting_documentation: str, optional
        :param next_action: Indicates the next action for completing the payment. Response only. One of the following values are - * 3d_verification - The next action is 3DS authentication. To simulate 3DS authentication in the sandbox, see Simulating 3DS Authentication. Relevant only to card payments. * pending_capture - The next action is pending the capture of the amount. Relevant only to card payments when the amount is not zero. * pending_confirmation - The next action is pending the confirmation for the payment. Relevant to all payment methods excluding card payment. * not_applicable - The payment has completed or the next action is not relevant., defaults to None
        :type next_action: NextAction, optional
        """
        self.bic_swift = self._define_str("bic_swift", bic_swift, nullable=True)
        self.account_last4 = self._define_str(
            "account_last4", account_last4, nullable=True
        )
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.type_ = self._define_str("type_", type_, nullable=True)
        self.category = (
            self._enum_matching(category, Category.list(), "category")
            if category
            else None
        )
        self.metadata = metadata
        self.image = self._define_str("image", image, nullable=True)
        self.webhook_url = self._define_str("webhook_url", webhook_url, nullable=True)
        self.supporting_documentation = self._define_str(
            "supporting_documentation", supporting_documentation, nullable=True
        )
        self.next_action = (
            self._enum_matching(next_action, NextAction.list(), "next_action")
            if next_action
            else None
        )


@JsonMap({"id_": "id", "type_": "type"})
class CustomerPaymentMethod2(BaseModel):
    """CustomerPaymentMethod2

    :param last_name: Customer's last name., defaults to None
    :type last_name: str, optional
    :param first_name: Customer's first name., defaults to None
    :type first_name: str, optional
    :param id_: ID of the Payment Method object. String starting with **card_** or **other_**., defaults to None
    :type id_: str, optional
    :param type_: Name of the payment method type. For example, **it_visa_card**. To get a list of payment methods for a country, use 'List Payment Methods by Country'., defaults to None
    :type type_: str, optional
    :param category: category, defaults to None
    :type category: Category, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param image: A URL to the image of the icon for the type of payment method. Response only, defaults to None
    :type image: str, optional
    :param webhook_url: Reserved. Response only, defaults to None
    :type webhook_url: str, optional
    :param supporting_documentation: Reserved. Response only., defaults to None
    :type supporting_documentation: str, optional
    :param next_action: Indicates the next action for completing the payment. Response only. One of the following values are - * 3d_verification - The next action is 3DS authentication. To simulate 3DS authentication in the sandbox, see Simulating 3DS Authentication. Relevant only to card payments. * pending_capture - The next action is pending the capture of the amount. Relevant only to card payments when the amount is not zero. * pending_confirmation - The next action is pending the confirmation for the payment. Relevant to all payment methods excluding card payment. * not_applicable - The payment has completed or the next action is not relevant., defaults to None
    :type next_action: NextAction, optional
    """

    def __init__(
        self,
        last_name: str = None,
        first_name: str = None,
        id_: str = None,
        type_: str = None,
        category: Category = None,
        metadata: dict = None,
        image: str = None,
        webhook_url: str = None,
        supporting_documentation: str = None,
        next_action: NextAction = None,
    ):
        """CustomerPaymentMethod2

        :param last_name: Customer's last name., defaults to None
        :type last_name: str, optional
        :param first_name: Customer's first name., defaults to None
        :type first_name: str, optional
        :param id_: ID of the Payment Method object. String starting with **card_** or **other_**., defaults to None
        :type id_: str, optional
        :param type_: Name of the payment method type. For example, **it_visa_card**. To get a list of payment methods for a country, use 'List Payment Methods by Country'., defaults to None
        :type type_: str, optional
        :param category: category, defaults to None
        :type category: Category, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param image: A URL to the image of the icon for the type of payment method. Response only, defaults to None
        :type image: str, optional
        :param webhook_url: Reserved. Response only, defaults to None
        :type webhook_url: str, optional
        :param supporting_documentation: Reserved. Response only., defaults to None
        :type supporting_documentation: str, optional
        :param next_action: Indicates the next action for completing the payment. Response only. One of the following values are - * 3d_verification - The next action is 3DS authentication. To simulate 3DS authentication in the sandbox, see Simulating 3DS Authentication. Relevant only to card payments. * pending_capture - The next action is pending the capture of the amount. Relevant only to card payments when the amount is not zero. * pending_confirmation - The next action is pending the confirmation for the payment. Relevant to all payment methods excluding card payment. * not_applicable - The payment has completed or the next action is not relevant., defaults to None
        :type next_action: NextAction, optional
        """
        self.last_name = self._define_str("last_name", last_name, nullable=True)
        self.first_name = self._define_str("first_name", first_name, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.type_ = self._define_str("type_", type_, nullable=True)
        self.category = (
            self._enum_matching(category, Category.list(), "category")
            if category
            else None
        )
        self.metadata = metadata
        self.image = self._define_str("image", image, nullable=True)
        self.webhook_url = self._define_str("webhook_url", webhook_url, nullable=True)
        self.supporting_documentation = self._define_str(
            "supporting_documentation", supporting_documentation, nullable=True
        )
        self.next_action = (
            self._enum_matching(next_action, NextAction.list(), "next_action")
            if next_action
            else None
        )


@JsonMap({"id_": "id", "type_": "type"})
class CustomerPaymentMethod3(BaseModel):
    """CustomerPaymentMethod3

    :param last4: Last four digits of the card or IBAN (International Bank Account Number)., defaults to None
    :type last4: str, optional
    :param acs_check: Results of the Access Control Server (ACS) check. Possible values: * pass *fail * unavailable *unchecked * Relevant to cards., defaults to None
    :type acs_check: str, optional
    :param cvv_check: Verification of the card’s CVV. Valid values: * pass * fail * unavailable *  unchecked, defaults to None
    :type cvv_check: str, optional
    :param bin_details: Bank Identification Number (BIN) details. Read-only. Object containing the following fields - * bin_number - BIN number * country - The two-letter ISO 3166-1 ALPHA-2 code for the country. Uppercase. * funding - Type of card funding. One of the following [credit, debit, prepaid, unknown] * bank - Name of the issuing bank. Relevant to cards, defaults to None
    :type bin_details: BinDetails, optional
    :param expiration_year: Year of expiration., defaults to None
    :type expiration_year: str, optional
    :param expiration_month: Month of expiration., defaults to None
    :type expiration_month: str, optional
    :param fingerprint_token: Hash of the card number, expiration date and CVV., defaults to None
    :type fingerprint_token: str, optional
    :param id_: ID of the Payment Method object. String starting with **card_** or **other_**., defaults to None
    :type id_: str, optional
    :param type_: Name of the payment method type. For example, **it_visa_card**. To get a list of payment methods for a country, use 'List Payment Methods by Country'., defaults to None
    :type type_: str, optional
    :param category: category, defaults to None
    :type category: Category, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param image: A URL to the image of the icon for the type of payment method. Response only, defaults to None
    :type image: str, optional
    :param webhook_url: Reserved. Response only, defaults to None
    :type webhook_url: str, optional
    :param supporting_documentation: Reserved. Response only., defaults to None
    :type supporting_documentation: str, optional
    :param next_action: Indicates the next action for completing the payment. Response only. One of the following values are - * 3d_verification - The next action is 3DS authentication. To simulate 3DS authentication in the sandbox, see Simulating 3DS Authentication. Relevant only to card payments. * pending_capture - The next action is pending the capture of the amount. Relevant only to card payments when the amount is not zero. * pending_confirmation - The next action is pending the confirmation for the payment. Relevant to all payment methods excluding card payment. * not_applicable - The payment has completed or the next action is not relevant., defaults to None
    :type next_action: NextAction, optional
    """

    def __init__(
        self,
        last4: str = None,
        acs_check: str = None,
        cvv_check: str = None,
        bin_details: BinDetails = None,
        expiration_year: str = None,
        expiration_month: str = None,
        fingerprint_token: str = None,
        id_: str = None,
        type_: str = None,
        category: Category = None,
        metadata: dict = None,
        image: str = None,
        webhook_url: str = None,
        supporting_documentation: str = None,
        next_action: NextAction = None,
    ):
        """CustomerPaymentMethod3

        :param last4: Last four digits of the card or IBAN (International Bank Account Number)., defaults to None
        :type last4: str, optional
        :param acs_check: Results of the Access Control Server (ACS) check. Possible values: * pass *fail * unavailable *unchecked * Relevant to cards., defaults to None
        :type acs_check: str, optional
        :param cvv_check: Verification of the card’s CVV. Valid values: * pass * fail * unavailable *  unchecked, defaults to None
        :type cvv_check: str, optional
        :param bin_details: Bank Identification Number (BIN) details. Read-only. Object containing the following fields - * bin_number - BIN number * country - The two-letter ISO 3166-1 ALPHA-2 code for the country. Uppercase. * funding - Type of card funding. One of the following [credit, debit, prepaid, unknown] * bank - Name of the issuing bank. Relevant to cards, defaults to None
        :type bin_details: BinDetails, optional
        :param expiration_year: Year of expiration., defaults to None
        :type expiration_year: str, optional
        :param expiration_month: Month of expiration., defaults to None
        :type expiration_month: str, optional
        :param fingerprint_token: Hash of the card number, expiration date and CVV., defaults to None
        :type fingerprint_token: str, optional
        :param id_: ID of the Payment Method object. String starting with **card_** or **other_**., defaults to None
        :type id_: str, optional
        :param type_: Name of the payment method type. For example, **it_visa_card**. To get a list of payment methods for a country, use 'List Payment Methods by Country'., defaults to None
        :type type_: str, optional
        :param category: category, defaults to None
        :type category: Category, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param image: A URL to the image of the icon for the type of payment method. Response only, defaults to None
        :type image: str, optional
        :param webhook_url: Reserved. Response only, defaults to None
        :type webhook_url: str, optional
        :param supporting_documentation: Reserved. Response only., defaults to None
        :type supporting_documentation: str, optional
        :param next_action: Indicates the next action for completing the payment. Response only. One of the following values are - * 3d_verification - The next action is 3DS authentication. To simulate 3DS authentication in the sandbox, see Simulating 3DS Authentication. Relevant only to card payments. * pending_capture - The next action is pending the capture of the amount. Relevant only to card payments when the amount is not zero. * pending_confirmation - The next action is pending the confirmation for the payment. Relevant to all payment methods excluding card payment. * not_applicable - The payment has completed or the next action is not relevant., defaults to None
        :type next_action: NextAction, optional
        """
        self.last4 = self._define_str("last4", last4, nullable=True)
        self.acs_check = self._define_str("acs_check", acs_check, nullable=True)
        self.cvv_check = self._define_str("cvv_check", cvv_check, nullable=True)
        self.bin_details = self._define_object(bin_details, BinDetails)
        self.expiration_year = self._define_str(
            "expiration_year", expiration_year, nullable=True
        )
        self.expiration_month = self._define_str(
            "expiration_month", expiration_month, nullable=True
        )
        self.fingerprint_token = self._define_str(
            "fingerprint_token", fingerprint_token, nullable=True
        )
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.type_ = self._define_str("type_", type_, nullable=True)
        self.category = (
            self._enum_matching(category, Category.list(), "category")
            if category
            else None
        )
        self.metadata = metadata
        self.image = self._define_str("image", image, nullable=True)
        self.webhook_url = self._define_str("webhook_url", webhook_url, nullable=True)
        self.supporting_documentation = self._define_str(
            "supporting_documentation", supporting_documentation, nullable=True
        )
        self.next_action = (
            self._enum_matching(next_action, NextAction.list(), "next_action")
            if next_action
            else None
        )


class CustomerPaymentMethodGuard(OneOfBaseModel):
    class_list = {
        "CustomerPaymentMethod1": CustomerPaymentMethod1,
        "CustomerPaymentMethod2": CustomerPaymentMethod2,
        "CustomerPaymentMethod3": CustomerPaymentMethod3,
    }


CustomerPaymentMethod = Union[
    CustomerPaymentMethod1, CustomerPaymentMethod2, CustomerPaymentMethod3
]



from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import (
    InlineResponse200_30,
    InlineResponse200_31,
    InlineResponse200_32,
    RefundsCompleteBody,
    RefundsGroupPaymentsBody,
    RefundsRefundIdBody,
    V1RefundsBody,
)


class RefundService(BaseService):

    @cast_models
    def all_refunds(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        ending_before: str = None,
        limit: str = None,
        starting_after: str = None,
        idempotency: str = None,
    ) -> InlineResponse200_30:
        """Retrieve a list of all refunds.

        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param ending_before: The ID of the refund created after the last refund you want to retrieve. String starting with refund_., defaults to None
        :type ending_before: str, optional
        :param limit: The maximum number of refunds to return. Range, 1-100. Default is 10., defaults to None
        :type limit: str, optional
        :param starting_after: The ID of the refund created before the first refund you want to retrieve. String starting with **refund_**., defaults to None
        :type starting_after: str, optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List Contacts for a Rapyd Wallet
        :rtype: InlineResponse200_30
        """

        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(ending_before)
        Validator(str).is_optional().validate(limit)
        Validator(str).is_optional().validate(starting_after)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/refunds", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_query("ending_before", ending_before, explode=False)
            .add_query("limit", limit, explode=False)
            .add_query("starting_after", starting_after, explode=False)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_30._unmap(response)

    @cast_models
    def request_total_create_refund(
        self,
        request_body: V1RefundsBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_31:
        """Refund of a payment. The refund is credited against a specific payment. The money is returned to the payment method that was used for the payment, and the currency is the same as what was used in the payment. If the action of a third party is not required, this method triggers the Refund Completed webhook. This webhook contains the same information as the response.

        :param request_body: The request body.
        :type request_body: V1RefundsBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List Contacts for a Rapyd Wallet
        :rtype: InlineResponse200_31
        """

        Validator(V1RefundsBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/refunds", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_31._unmap(response)

    @cast_models
    def simulate_complete_refund(
        self,
        request_body: RefundsCompleteBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_31:
        """Simulate the action of a third party that is required for completing the refund process. Relevant to sandbox. Use this method when a payment was made with a payment method such as cash, bank redirect or bank transfer, and the payment was completed by an action taken by the customer.

        :param request_body: The request body.
        :type request_body: RefundsCompleteBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List Contacts for a Rapyd Wallet
        :rtype: InlineResponse200_31
        """

        Validator(RefundsCompleteBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/refunds/complete", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_31._unmap(response)

    @cast_models
    def refund_group_payment(
        self, request_body: RefundsGroupPaymentsBody
    ) -> InlineResponse200_32:
        """Refund a group payment when the status of the group payment is closed. The refund is credited against a specific group payment. The money is returned to the payment methods that were used for the payment. If the action of a third party is not required, this method triggers the Refund Completed webhook for each payment method.

        :param request_body: The request body.
        :type request_body: RefundsGroupPaymentsBody
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List Contacts for a Rapyd Wallet
        :rtype: InlineResponse200_32
        """

        Validator(RefundsGroupPaymentsBody).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/refunds/group_payments", self.get_default_headers()
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_32._unmap(response)

    @cast_models
    def refund_by_token(
        self,
        refund_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_31:
        """Retrieve the details of a refund object.

        :param refund_id: ID of the 'refund' object you want to retrieve. String starting with **refund_**.
        :type refund_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List Contacts for a Rapyd Wallet
        :rtype: InlineResponse200_31
        """

        Validator(str).validate(refund_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/refunds/{{refundId}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("refundId", refund_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_31._unmap(response)

    @cast_models
    def update_refund(
        self,
        request_body: RefundsRefundIdBody,
        refund_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_31:
        """Change or modify the metadata in a refund object.

        :param request_body: The request body.
        :type request_body: RefundsRefundIdBody
        :param refund_id: ID of the 'refund' object you want to retrieve. String starting with **refund_**.
        :type refund_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List Contacts for a Rapyd Wallet
        :rtype: InlineResponse200_31
        """

        Validator(RefundsRefundIdBody).validate(request_body)
        Validator(str).validate(refund_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/refunds/{{refundId}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("refundId", refund_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_31._unmap(response)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class PlanTransformUsage(BaseModel):
    """Defines the transformation that is applied to the reported usage before the billed price is computed. The transformation divides the quantity by the divisor specified in divide_by, then rounds up or down according to the setting in round. Relevant when billing_scheme is set to per_unit.

    :param divide_by: Indicates the divisor in the transformation calculation. Integer. Default is 1., defaults to None
    :type divide_by: float, optional
    :param round: Indicates whether the reported number of units should be rounded up or down to the next whole quantity specified in divide_by. String. Default is up., defaults to None
    :type round: str, optional
    """

    def __init__(self, divide_by: float = None, round: str = None):
        """Defines the transformation that is applied to the reported usage before the billed price is computed. The transformation divides the quantity by the divisor specified in divide_by, then rounds up or down according to the setting in round. Relevant when billing_scheme is set to per_unit.

        :param divide_by: Indicates the divisor in the transformation calculation. Integer. Default is 1., defaults to None
        :type divide_by: float, optional
        :param round: Indicates whether the reported number of units should be rounded up or down to the next whole quantity specified in divide_by. String. Default is up., defaults to None
        :type round: str, optional
        """
        self.divide_by = self._define_number("divide_by", divide_by, nullable=True)
        self.round = self._define_str("round", round, nullable=True)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class VirtualAccountsTransactionsBody(BaseModel):
    """VirtualAccountsTransactionsBody

    :param amount: The amount of the transaction, in units of the currency defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015. If the amount is a whole number, use an integer and not a decimal.
    :type amount: str
    :param currency: Currency of the transaction. Three-letter ISO 4217 code. Must be the same as the currency of the virtual account.
    :type currency: str
    :param issued_bank_account: ID of the Virtual Account Number object. String starting with issuing_
    :type issued_bank_account: str
    """

    def __init__(self, amount: str, currency: str, issued_bank_account: str):
        """VirtualAccountsTransactionsBody

        :param amount: The amount of the transaction, in units of the currency defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015. If the amount is a whole number, use an integer and not a decimal.
        :type amount: str
        :param currency: Currency of the transaction. Three-letter ISO 4217 code. Must be the same as the currency of the virtual account.
        :type currency: str
        :param issued_bank_account: ID of the Virtual Account Number object. String starting with issuing_
        :type issued_bank_account: str
        """
        self.amount = amount
        self.currency = currency
        self.issued_bank_account = issued_bank_account



from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import InlineResponse200, InlineResponse200_1


class PaymentMethodTypeService(BaseService):

    @cast_models
    def get_payment_methods_types_by_country(
        self,
        country_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        currency: str = None,
        idempotency: str = None,
    ) -> InlineResponse200:
        """Retrieve a list of all payment methods available for a country. You can filter the results by specifying the `currency` query parameter

        :param country_id: Two-letter ISO 3166-1 ALPHA-2 code for the country. Uppercase.
        :type country_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param currency: currency, defaults to None
        :type currency: str, optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: SUCCESS
        :rtype: InlineResponse200
        """

        Validator(str).pattern(
            "Name of the country. Two-letter ISO 3166-1 alpha-2 code."
        ).validate(country_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().pattern(
            "/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/"
        ).validate(currency)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payment_methods/countries/{{countryId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("countryId", country_id)
            .add_query("currency", currency)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200._unmap(response)

    @cast_models
    def get_payment_method_type_required_fields(
        self,
        pmt_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_1:
        """Retrieve the required fields for a payment method. The fields are returned as a list of objects. The name of each field appears in the `name` field of the response.

        :param pmt_id: Payment method type Id
        :type pmt_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: SUCCESS
        :rtype: InlineResponse200_1
        """

        Validator(str).validate(pmt_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payment_methods/{{pmtId}}/required_fields",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("pmtId", pmt_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_1._unmap(response)



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .address_1 import Address1


@JsonMap({})
class PaymentsPaymentIdBody(BaseModel):
    """PaymentsPaymentIdBody

    :param address: address associated with this specific Rapyd entity Payment/Customer etc., defaults to None
    :type address: Address1, optional
    :param description: Description of the payment., defaults to None
    :type description: str, optional
    :param escrow: To release escrow funds immediately, set to false. If there are multiple sellers, the funds are released to all of them. Relevant to payments with escrow., defaults to None
    :type escrow: bool, optional
    :param escrow_release_days: Defines the number of days after the creation of the payment that the funds are automatically released. Relevant when escrow is true., defaults to None
    :type escrow_release_days: int, optional
    :param initiation_type: Category of transaction initiation type., defaults to None
    :type initiation_type: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param receipt_email: Email address that the receipt for this transaction is sent to., defaults to None
    :type receipt_email: str, optional
    """

    def __init__(
        self,
        address: Address1 = None,
        description: str = None,
        escrow: bool = None,
        escrow_release_days: int = None,
        initiation_type: str = None,
        metadata: dict = None,
        receipt_email: str = None,
    ):
        """PaymentsPaymentIdBody

        :param address: address associated with this specific Rapyd entity Payment/Customer etc., defaults to None
        :type address: Address1, optional
        :param description: Description of the payment., defaults to None
        :type description: str, optional
        :param escrow: To release escrow funds immediately, set to false. If there are multiple sellers, the funds are released to all of them. Relevant to payments with escrow., defaults to None
        :type escrow: bool, optional
        :param escrow_release_days: Defines the number of days after the creation of the payment that the funds are automatically released. Relevant when escrow is true., defaults to None
        :type escrow_release_days: int, optional
        :param initiation_type: Category of transaction initiation type., defaults to None
        :type initiation_type: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param receipt_email: Email address that the receipt for this transaction is sent to., defaults to None
        :type receipt_email: str, optional
        """
        self.address = self._define_object(address, Address1)
        self.description = self._define_str("description", description, nullable=True)
        self.escrow = escrow
        self.escrow_release_days = self._define_number(
            "escrow_release_days", escrow_release_days, nullable=True
        )
        self.initiation_type = self._define_str(
            "initiation_type", initiation_type, nullable=True
        )
        self.metadata = metadata
        self.receipt_email = self._define_str(
            "receipt_email", receipt_email, nullable=True
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class InlineResponse200_119DataRegistrationInfo(BaseModel):
    """Registration information in the card network's termination database about the merchant and the acquirer that listed the merchant.

    :param contract_end_date: End date of the merchant's contract with the acquirer., defaults to None
    :type contract_end_date: float, optional
    :param contract_start_date: start date of the merchant's contract with the acquirer., defaults to None
    :type contract_start_date: float, optional
    :param primary_registration_reason: Reason that the merchant's contract with the acquirer was terminated., defaults to None
    :type primary_registration_reason: str, optional
    :param registered_by_acquirer_id: ID of the acquirer that listed the merchant., defaults to None
    :type registered_by_acquirer_id: str, optional
    :param registered_by_acquirer_name: Name of the acquirer that listed the merchant., defaults to None
    :type registered_by_acquirer_name: str, optional
    :param registered_by_acquirer_region: Region of the acquirer that listed the merchant., defaults to None
    :type registered_by_acquirer_region: str, optional
    """

    def __init__(
        self,
        contract_end_date: float = None,
        contract_start_date: float = None,
        primary_registration_reason: str = None,
        registered_by_acquirer_id: str = None,
        registered_by_acquirer_name: str = None,
        registered_by_acquirer_region: str = None,
    ):
        """Registration information in the card network's termination database about the merchant and the acquirer that listed the merchant.

        :param contract_end_date: End date of the merchant's contract with the acquirer., defaults to None
        :type contract_end_date: float, optional
        :param contract_start_date: start date of the merchant's contract with the acquirer., defaults to None
        :type contract_start_date: float, optional
        :param primary_registration_reason: Reason that the merchant's contract with the acquirer was terminated., defaults to None
        :type primary_registration_reason: str, optional
        :param registered_by_acquirer_id: ID of the acquirer that listed the merchant., defaults to None
        :type registered_by_acquirer_id: str, optional
        :param registered_by_acquirer_name: Name of the acquirer that listed the merchant., defaults to None
        :type registered_by_acquirer_name: str, optional
        :param registered_by_acquirer_region: Region of the acquirer that listed the merchant., defaults to None
        :type registered_by_acquirer_region: str, optional
        """
        self.contract_end_date = self._define_number(
            "contract_end_date", contract_end_date, nullable=True
        )
        self.contract_start_date = self._define_number(
            "contract_start_date", contract_start_date, nullable=True
        )
        self.primary_registration_reason = self._define_str(
            "primary_registration_reason", primary_registration_reason, nullable=True
        )
        self.registered_by_acquirer_id = self._define_str(
            "registered_by_acquirer_id", registered_by_acquirer_id, nullable=True
        )
        self.registered_by_acquirer_name = self._define_str(
            "registered_by_acquirer_name", registered_by_acquirer_name, nullable=True
        )
        self.registered_by_acquirer_region = self._define_str(
            "registered_by_acquirer_region",
            registered_by_acquirer_region,
            nullable=True,
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class PaymentParams(BaseModel):
    """PaymentParams

    :param complete_payment_url: URL where the customer is redirected after successfully completing an operation on a hosted page. Does not support localhost URLs., defaults to None
    :type complete_payment_url: str, optional
    :param error_payment_url: URL where the customer is redirected if an error occurs during or after an operation on a hosted page. Does not support localhost URLs., defaults to None
    :type error_payment_url: str, optional
    """

    def __init__(self, complete_payment_url: str = None, error_payment_url: str = None):
        """PaymentParams

        :param complete_payment_url: URL where the customer is redirected after successfully completing an operation on a hosted page. Does not support localhost URLs., defaults to None
        :type complete_payment_url: str, optional
        :param error_payment_url: URL where the customer is redirected if an error occurs during or after an operation on a hosted page. Does not support localhost URLs., defaults to None
        :type error_payment_url: str, optional
        """
        self.complete_payment_url = self._define_str(
            "complete_payment_url", complete_payment_url, nullable=True
        )
        self.error_payment_url = self._define_str(
            "error_payment_url", error_payment_url, nullable=True
        )



from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import (
    Coupon,
    InlineResponse200_50,
    InlineResponse200_51,
    InlineResponse200_52,
)


class CouponService(BaseService):

    @cast_models
    def list_coupon(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        starting_after: str = None,
        ending_before: str = None,
        limit: str = None,
        idempotency: str = None,
    ) -> InlineResponse200_50:
        """Retrieve a list of all coupons.

        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param starting_after: The ID of the coupon created before the first coupon you want to retrieve., defaults to None
        :type starting_after: str, optional
        :param ending_before: The ID of the coupon created after the last coupon you want to       retrieve., defaults to None
        :type ending_before: str, optional
        :param limit: The maximum number of coupons to return. Range is 1-100. Default is 10, defaults to None
        :type limit: str, optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List of coupons
        :rtype: InlineResponse200_50
        """

        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(starting_after)
        Validator(str).is_optional().validate(ending_before)
        Validator(str).is_optional().pattern("([1-9]|[1-9]\d|100)").validate(limit)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/coupons", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_query("starting_after", starting_after)
            .add_query("ending_before", ending_before)
            .add_query("limit", limit)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_50._unmap(response)

    @cast_models
    def create_coupon(
        self,
        request_body: Coupon,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_51:
        """create a coupon

        :param request_body: The request body.
        :type request_body: Coupon
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Coupon was created
        :rtype: InlineResponse200_51
        """

        Validator(Coupon).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/coupons", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_51._unmap(response)

    @cast_models
    def retrieve_coupon(
        self,
        coupon_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_51:
        """Retrieve a coupon

        :param coupon_id: coupon ID. String starting with **coupon_**.
        :type coupon_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Get Coupon details By coupon Id
        :rtype: InlineResponse200_51
        """

        Validator(str).validate(coupon_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/coupons/{{couponId}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("couponId", coupon_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_51._unmap(response)

    @cast_models
    def update_coupon(
        self,
        request_body: Coupon,
        coupon_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_51:
        """Modify the metadata of a coupon with a coupon ID in Path

        :param request_body: The request body.
        :type request_body: Coupon
        :param coupon_id: coupon ID. String starting with **coupon_**.
        :type coupon_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Coupon was updated.
        :rtype: InlineResponse200_51
        """

        Validator(Coupon).validate(request_body)
        Validator(str).validate(coupon_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/coupons/{{couponId}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("couponId", coupon_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_51._unmap(response)

    @cast_models
    def delete_coupon(
        self,
        coupon_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_52:
        """Delete a coupon from the Rapyd platform. Deleting a coupon removes it from all customers and subscriptions, but does not affect invoices and payment authorizations that have already been calculated.

        :param coupon_id: Coupon ID. String starting with **coupon_**.
        :type coupon_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Coupon was deleted.
        :rtype: InlineResponse200_52
        """

        Validator(str).validate(coupon_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/coupons/{{couponId}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("couponId", coupon_id)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_52._unmap(response)



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .inline_response_200_119_data_query_info_queried_merchant import (
    InlineResponse200_119DataQueryInfoQueriedMerchant,
)


@JsonMap({})
class InlineResponse200_119DataQueryInfo(BaseModel):
    """asd

    :param partner_merchant_reference: ID of the merchant, defined by the partner., defaults to None
    :type partner_merchant_reference: str, optional
    :param partner_query_reference: Unique ID of the query request, defined by the partner., defaults to None
    :type partner_query_reference: str, optional
    :param queried_merchant: Details of the merchant who is the subject of the query., defaults to None
    :type queried_merchant: InlineResponse200_119DataQueryInfoQueriedMerchant, optional
    """

    def __init__(
        self,
        partner_merchant_reference: str = None,
        partner_query_reference: str = None,
        queried_merchant: InlineResponse200_119DataQueryInfoQueriedMerchant = None,
    ):
        """asd

        :param partner_merchant_reference: ID of the merchant, defined by the partner., defaults to None
        :type partner_merchant_reference: str, optional
        :param partner_query_reference: Unique ID of the query request, defined by the partner., defaults to None
        :type partner_query_reference: str, optional
        :param queried_merchant: Details of the merchant who is the subject of the query., defaults to None
        :type queried_merchant: InlineResponse200_119DataQueryInfoQueriedMerchant, optional
        """
        self.partner_merchant_reference = self._define_str(
            "partner_merchant_reference", partner_merchant_reference, nullable=True
        )
        self.partner_query_reference = self._define_str(
            "partner_query_reference", partner_query_reference, nullable=True
        )
        self.queried_merchant = self._define_object(
            queried_merchant, InlineResponse200_119DataQueryInfoQueriedMerchant
        )



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .v1cnltermination_query_search_criteria import V1cnlterminationQuerySearchCriteria
from .v1cnltermination_query_queried_merchant import (
    V1cnlterminationQueryQueriedMerchant,
)


@JsonMap({})
class CnlTerminationQueryBody(BaseModel):
    """CnlTerminationQueryBody

    :param partner_merchant_reference: ID of the merchant, defined by the partner.\<BR\>Length: 0-60
    :type partner_merchant_reference: str
    :param partner_query_reference: Unique ID of the query request, defined by the partner.\<BR\>Length: 0-60
    :type partner_query_reference: str
    :param search_criteria: Specifies search criteria for the query., defaults to None
    :type search_criteria: V1cnlterminationQuerySearchCriteria, optional
    :param queried_merchant: Information about the merchant who is the subject of the query.
    :type queried_merchant: V1cnlterminationQueryQueriedMerchant
    """

    def __init__(
        self,
        partner_merchant_reference: str,
        partner_query_reference: str,
        queried_merchant: V1cnlterminationQueryQueriedMerchant,
        search_criteria: V1cnlterminationQuerySearchCriteria = None,
    ):
        """CnlTerminationQueryBody

        :param partner_merchant_reference: ID of the merchant, defined by the partner.\<BR\>Length: 0-60
        :type partner_merchant_reference: str
        :param partner_query_reference: Unique ID of the query request, defined by the partner.\<BR\>Length: 0-60
        :type partner_query_reference: str
        :param search_criteria: Specifies search criteria for the query., defaults to None
        :type search_criteria: V1cnlterminationQuerySearchCriteria, optional
        :param queried_merchant: Information about the merchant who is the subject of the query.
        :type queried_merchant: V1cnlterminationQueryQueriedMerchant
        """
        self.partner_merchant_reference = partner_merchant_reference
        self.partner_query_reference = partner_query_reference
        self.search_criteria = self._define_object(
            search_criteria, V1cnlterminationQuerySearchCriteria
        )
        self.queried_merchant = self._define_object(
            queried_merchant, V1cnlterminationQueryQueriedMerchant
        )
